#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL_ttf.h>
#include "background.h"
#include "ask.h"
#include "input.h"
#include "ship.h"
#include "files.h"
#include "common.h"
#include "options.h"
#include "main.h"

#define DEFAULT_PTSIZE	36
#define DEFAULT_SPACE	20
#define OPTIONS_MENU_ITEMS_COUNT    6
#define CONTROLS_MENU_ITEMS_COUNT   9
#define BUFF_MAX_MENU_ITEMS_COUNT   12
#define BUFF_MAX_ITEM_SIZE	    100

const char *menu[] = {
    "New game",
    "Options ...",
    "Quit",
    0
};

const enum menu_ids menu_id[] = {
    menu_new_game,
    menu_options,
    menu_quit,
    menu_nothing
};

int players = 2;
extern int maximum_ships_count;

const char *options_menu_skel[OPTIONS_MENU_ITEMS_COUNT] = {
    "Number of players: %d",
    "Screen resolution: %dx%d",
    "Toggle fullscreen",
    "Controls ...",
    "Return to main menu",
    0
};

const char *controls_menu_skel[CONTROLS_MENU_ITEMS_COUNT] = {
    "Input method: %s",
    "Speed up control: %s",
    "Speed down control: %s",
    "Rotate left control: %s",
    "Rotate right control: %s",
    "Shot control: %s",
    "Blast control: %s",
    "Return to previous menu",
    0
};

const char *controls_menu_skel_joy[CONTROLS_MENU_ITEMS_COUNT - 2] = {
    "Input method: %s",
    "Joystick: %s",
    "Shot button: %hhu",
    "Blast button: %hhu",
    "Calibrate joystick",
    "Return to previous menu",
    0
};

const char *input_type_str[] = {
    "None",
    "Keyboard",
    "Joystick",
    0
};

char options_menu_buff[BUFF_MAX_MENU_ITEMS_COUNT][BUFF_MAX_ITEM_SIZE], *options_array[BUFF_MAX_MENU_ITEMS_COUNT];
enum menu_ids options_menu_id_buff[BUFF_MAX_MENU_ITEMS_COUNT];

const enum menu_ids options_menu_id[] = {
    menu_players,
    menu_screen_size,
    menu_fullscreen,
    menu_controls,
    menu_return,
    menu_nothing
};

TTF_Font *font;
menu_item_type *menu_items = NULL;
int items = 0;

void
init_menu()
{
    font = TTF_OpenFont(get_file(DEFAULT_FONT_NAME, font_file), DEFAULT_PTSIZE);
    menu_items = create_menu(menu, menu_id);
}

menu_item_type *
create_menu(const char *menu_str[], const enum menu_ids ids_for_menu_items[])
{
    SDL_Color white = { 0xFF, 0xFF, 0xFF, 0 };
    SDL_Color light_gray = { 0xD0, 0xD0, 0xD0, 0 };
    SDL_Color black = { 0x00, 0x00, 0x00, 0 };
    int menu_height = 0, counter, start_y;
    menu_item_type *new_menu_items;

    while (menu_str[items]) items++;
    new_menu_items = (menu_item_type *)malloc(sizeof(menu_item_type) * items);
    memset((void *)new_menu_items, 0, sizeof(menu_item_type) * items);
    for (counter = 0; counter < items; counter++) {
	new_menu_items[counter].normal = TTF_RenderText_Shaded(font, menu_str[counter], light_gray, black);
	new_menu_items[counter].active = TTF_RenderText_Shaded(font, menu_str[counter], white, black);
	new_menu_items[counter].action_id = ids_for_menu_items[counter];
	SDL_SetColorKey(new_menu_items[counter].normal, SDL_SRCCOLORKEY, SDL_MapRGB(new_menu_items[counter].normal->format, 0, 0, 0));
	SDL_SetColorKey(new_menu_items[counter].active, SDL_SRCCOLORKEY, SDL_MapRGB(new_menu_items[counter].active->format, 0, 0, 0));
	new_menu_items[counter].item_rect.x = (screen_width - new_menu_items[counter].active->w) / 2;
	new_menu_items[counter].item_rect.w = new_menu_items[counter].active->w;
	new_menu_items[counter].item_rect.h = new_menu_items[counter].active->h;
	menu_height += new_menu_items[counter].active->h;
    }
    menu_height += (items - 1) * DEFAULT_SPACE;
    start_y = (screen_height - menu_height) / 2;
    for (counter = 0; counter < items; counter++) {
	new_menu_items[counter].item_rect.y = start_y;
	start_y += DEFAULT_SPACE + new_menu_items[counter].item_rect.h;
    }
    return new_menu_items;
}

void
delete_menu(menu_item_type *menu_to_delete)
{
    int counter;

    for (counter = 0; counter < items; counter++) {
	SDL_FreeSurface(menu_to_delete[counter].normal);
	SDL_FreeSurface(menu_to_delete[counter].active);
    }
    items = 0;
    free(menu_to_delete);
}

void
free_menu()
{
    if (menu_items)
	delete_menu(menu_items);
    TTF_CloseFont(font);
}

void
draw_menu_item(SDL_Surface *to, const int item_no, const int selected)
{
    draw_background(to, &menu_items[item_no].item_rect);
    draw_logo(to, &menu_items[item_no].item_rect);
    if (SDL_BlitSurface((selected ? menu_items[item_no].active : menu_items[item_no].normal),
		    NULL, to, &menu_items[item_no].item_rect) == 0) {
	    SDL_UpdateRects(to, 1, &menu_items[item_no].item_rect);
    } else {
	    fprintf(stderr, "Couldn't blit text to display: %s\n", SDL_GetError());
    }
}

void
clean_menu_item(SDL_Surface *to, const int item_no)
{
    draw_background(to, &menu_items[item_no].item_rect);
    draw_logo(to, &menu_items[item_no].item_rect);
    SDL_UpdateRects(to, 1, &menu_items[item_no].item_rect);
}

void
draw_menu(SDL_Surface *to)
{
    int counter;

    draw_background(to, NULL);
    draw_logo(to, NULL);
    SDL_Flip(to);
    for (counter = 0; counter < items; counter++)
	draw_menu_item(to, counter, 0);
}

int
start_menu(SDL_Surface *to)
{
    SDL_Event recv_event;
    int active = -1, counter;
    
    SDL_EventState(SDL_MOUSEMOTION, SDL_ENABLE);
    SDL_EventState(SDL_MOUSEBUTTONUP, SDL_ENABLE);
    SDL_EventState(SDL_MOUSEBUTTONDOWN, SDL_ENABLE);
    for (;;) {
	SDL_WaitEvent(&recv_event);
	switch(recv_event.type) {
	    case SDL_MOUSEMOTION:
		for (counter = 0; counter < items; counter++)
		    if (recv_event.motion.x >= menu_items[counter].item_rect.x &&
			    recv_event.motion.y >= menu_items[counter].item_rect.y &&
			    recv_event.motion.x < menu_items[counter].item_rect.x + menu_items[counter].item_rect.w &&
			    recv_event.motion.y < menu_items[counter].item_rect.y + menu_items[counter].item_rect.h)
			break;
		if (counter == active)
		    break;
		if (counter < items) {
		    if (active >= 0)
			draw_menu_item(to, active, 0);
		    draw_menu_item(to, counter, 1);
		    active = counter;
		} else {
		    if (active < 0)
			break;
		    draw_menu_item(to, active, 0);
		    active = -1;
		}
		break;
	    case SDL_MOUSEBUTTONDOWN:
		for (counter = 0; counter < items; counter++)
		    if (recv_event.button.x >= menu_items[counter].item_rect.x &&
			    recv_event.button.y >= menu_items[counter].item_rect.y &&
			    recv_event.button.x < menu_items[counter].item_rect.x + menu_items[counter].item_rect.w &&
			    recv_event.button.y < menu_items[counter].item_rect.y + menu_items[counter].item_rect.h)
			break;
		if (counter < items) {
		    switch (action_handler(&to, menu_items[counter].action_id, recv_event.button.button)) {
			case LOOP_CONT:
			    break;
			case LOOP_RESTART:
			    active = -1;
			    draw_menu(to);
			    break;
			case LOOP_QUIT:
			    return 0;
			case LOOP_REDRAW_ACTIVE:
			    draw_menu_item(to, counter, 1);
		    }
		}
		break;
	    case SDL_QUIT:
#if DEBUG >= EXTRA_DEBUG
		printf("Quit!\n");
#endif /* DEBUG */
		return 0;
	    default:
#if DEBUG >= EXTRA_DEBUG
		printf("Unhandled Event!\n");
#endif /* DEBUG */
		break;
	}
    }
}

char **create_options_menu()
{
    int counter;

    for (counter = 0; counter < OPTIONS_MENU_ITEMS_COUNT - 1; counter++) {
	switch (options_menu_id[counter]) {
	    case menu_players:
		sprintf(options_menu_buff[counter], options_menu_skel[counter], players);
		break;
	    case menu_screen_size:
		sprintf(options_menu_buff[counter], options_menu_skel[counter], screen_width, screen_height);
		break;
	    default:
		sprintf(options_menu_buff[counter], "%s", options_menu_skel[counter]);
		break;
	}
	options_array[counter] = options_menu_buff[counter];
    }
    options_array[counter] = NULL;
    return options_array;
}

char **create_players_menu()
{
    int counter;

    for (counter = 0; counter < maximum_ships_count; counter++) {
	sprintf(options_menu_buff[counter], "Player %d ...", counter + 1);
	options_array[counter] = options_menu_buff[counter];
    }
    options_array[counter] = options_menu_buff[counter];
    sprintf(options_menu_buff[counter++], "Return to previous menu");
    options_array[counter] = NULL;
    return options_array;
}

char **create_controls_menu(const int which_player)
{
    int counter, end;

    end = CONTROLS_MENU_ITEMS_COUNT - (get_players_input_method(which_player) == joystick ? 3 : 1);
    for (counter = 0; counter < end; counter++) {
	switch (counter) {
	    case 0:
		sprintf(options_menu_buff[counter], controls_menu_skel[counter], get_input_method_desc(get_players_input_method(which_player)));
		break;
	    case speed_up:
	    case speed_down:
	    case rotate_left:
	    case rotate_right:
	    case fire:
	    case fire_blast:
		switch (get_players_input_method(which_player)) {
		    case no_input:
			sprintf(options_menu_buff[counter], controls_menu_skel[counter], "None");
			break;
		    case keyboard:
			sprintf(options_menu_buff[counter], controls_menu_skel[counter], SDL_GetKeyName(get_action_key(which_player, counter)));
			break;
		    case joystick:
			switch (counter) {
			    case speed_up:
				if (is_joystick_active(which_player))
				    sprintf(options_menu_buff[counter], controls_menu_skel_joy[counter], SDL_JoystickName(get_joystick(which_player)));
				else
				    sprintf(options_menu_buff[counter], controls_menu_skel_joy[counter], "None");
				break;
				sprintf(options_menu_buff[counter], controls_menu_skel_joy[counter]);
				break;
			    case speed_down:
			    case rotate_left:
				if (is_joystick_active(which_player))
				    sprintf(options_menu_buff[counter], controls_menu_skel_joy[counter], get_action_joy_button(which_player, fire + counter - rotate_left + 1) + 1);
				else
				    sprintf(options_menu_buff[counter], controls_menu_skel_joy[counter], 0);
				break;
			    case rotate_right:
			    case fire:
				sprintf(options_menu_buff[counter], "%s", controls_menu_skel_joy[counter]);
				break;
			}
			break;
		}
		break;
	    default:
		sprintf(options_menu_buff[counter], "%s", controls_menu_skel[counter]);
	}
	options_array[counter] = options_menu_buff[counter];
    }
    options_array[counter] = NULL;
    return options_array;
}

enum menu_ids *create_controls_menu_id(const int which_player)
{
    int counter, end;

    end = CONTROLS_MENU_ITEMS_COUNT - (get_players_input_method(which_player) == joystick ? 4 : 2);
    for (counter = 0; counter < end; counter++) {
	options_menu_id_buff[counter] = menu_contr_player_1_method + (menu_contr_player_2_method - menu_contr_player_1_method) * which_player + counter;
	if (counter > 0) {
	    switch (get_players_input_method(which_player)) {
		case no_input:
		    options_menu_id_buff[counter] = menu_nothing;
		    break;
		case joystick:
		    options_menu_id_buff[counter] += (menu_contr_player_1_joystick - menu_contr_player_1_up);
		    break;
		default:
		    break;
	    }
	}
    }
    options_menu_id_buff[counter] = menu_controls;
    return options_menu_id_buff;
}

int
action_handler(SDL_Surface **to, const enum menu_ids action_id, const Uint8 button)
{
    int counter, which_player, result, count;
    input_method input;
    player_actions which_action;
    Uint8 which_joystick;
    SDL_Rect **screen_resolutions;
    
    switch (action_id) {
	case menu_new_game:
	    SDL_EventState(SDL_MOUSEMOTION, SDL_DISABLE);
	    SDL_EventState(SDL_MOUSEBUTTONUP, SDL_DISABLE);
	    SDL_EventState(SDL_MOUSEBUTTONDOWN, SDL_DISABLE);
	    free_ships();
	    for (counter = 0; counter < players; counter++)
		create_ship(counter);
	    Run();
	    SDL_EventState(SDL_MOUSEMOTION, SDL_ENABLE);
	    SDL_EventState(SDL_MOUSEBUTTONUP, SDL_ENABLE);
	    SDL_EventState(SDL_MOUSEBUTTONDOWN, SDL_ENABLE);
	    return LOOP_RESTART;
	case menu_quit:
	    return LOOP_QUIT;
	case menu_options:
	    delete_menu(menu_items);
	    menu_items = create_menu((const char **)create_options_menu(), options_menu_id);
	    return LOOP_RESTART;
	case menu_controls:
	    delete_menu(menu_items);
	    for (counter = 0; counter < maximum_ships_count; counter++) {
		options_menu_id_buff[counter] = menu_contr_player_1 + counter;
	    }
	    options_menu_id_buff[counter] = menu_options;
	    menu_items = create_menu((const char **)create_players_menu(), options_menu_id_buff);
	    return LOOP_RESTART;
	case menu_contr_player_1:
	case menu_contr_player_2:
	case menu_contr_player_3:
	case menu_contr_player_4:
	    which_player = action_id - menu_contr_player_1;
	    delete_menu(menu_items);
	    menu_items = create_menu((const char **)create_controls_menu(which_player),
		    (const enum menu_ids *)create_controls_menu_id(which_player));
	    return LOOP_RESTART;
	case menu_contr_player_1_method:
	case menu_contr_player_2_method:
	    which_player = (action_id - menu_contr_player_1_method) / (menu_contr_player_2_method - menu_contr_player_1_method);
	    input = get_players_input_method(which_player);
	    if (button == SDL_BUTTON_LEFT) {
		input++;
		if (input > joystick)
		    input = no_input;
	    } else {
		if (button == SDL_BUTTON_RIGHT) {
		    input--;
		    if (input < no_input)
			input = joystick;
		} else
		    break;
	    }
	    set_players_input_method(which_player, input);
	    delete_menu(menu_items);
	    menu_items = create_menu((const char **)create_controls_menu(which_player),
		    (const enum menu_ids *)create_controls_menu_id(which_player));
	    return LOOP_RESTART;
	case menu_contr_player_1_up:
	case menu_contr_player_1_down:
	case menu_contr_player_1_left:
	case menu_contr_player_1_right:
	case menu_contr_player_1_blast:
	case menu_contr_player_1_shot:
	case menu_contr_player_2_up:
	case menu_contr_player_2_down:
	case menu_contr_player_2_left:
	case menu_contr_player_2_right:
	case menu_contr_player_2_blast:
	case menu_contr_player_2_shot:
	    which_player = (action_id - menu_contr_player_1_method) / (menu_contr_player_2_method - menu_contr_player_1_method);
	    which_action = (action_id - menu_contr_player_1_method) % (menu_contr_player_2_method - menu_contr_player_1_method);
	    switch (get_players_input_method(which_player)) {
		case no_input:
		    return LOOP_CONT;
		case keyboard:
		    set_action_key(which_player, which_action, get_key());
		    break;
		case joystick:
		    return LOOP_CONT;
	    }
	    clean_menu_item(*to, which_action);
	    delete_menu(menu_items);
	    menu_items = create_menu((const char **)create_controls_menu(which_player),
		    (const enum menu_ids *)create_controls_menu_id(which_player));
	    return LOOP_REDRAW_ACTIVE;
	case menu_contr_player_1_joystick:
	case menu_contr_player_2_joystick:
	    which_player = (action_id - menu_contr_player_1_joystick) / (menu_contr_player_2_method - menu_contr_player_1_method);
	    which_joystick = get_joystick(which_player);
	    if (button == SDL_BUTTON_LEFT) {
		which_joystick++;
		if (which_joystick >= SDL_NumJoysticks())
		    which_joystick = 0;
	    } else {
		if (button == SDL_BUTTON_RIGHT) {
		    if (which_joystick == 0)
			which_joystick = SDL_NumJoysticks() - 1;
		    else
			which_joystick--;
		} else
		    break;
	    }
	    set_joystick(which_player, which_joystick);
	    clean_menu_item(*to, 1);
	    delete_menu(menu_items);
	    menu_items = create_menu((const char **)create_controls_menu(which_player),
		    (const enum menu_ids *)create_controls_menu_id(which_player));
	    return LOOP_REDRAW_ACTIVE;
	case menu_contr_player_1_shot_button:
	case menu_contr_player_1_blast_button:
	case menu_contr_player_2_shot_button:
	case menu_contr_player_2_blast_button:
	    which_player = (action_id - menu_contr_player_1_joystick) / (menu_contr_player_2_method - menu_contr_player_1_method);
	    which_joystick = get_joystick(which_player);
	    which_action = (action_id - menu_contr_player_1_joystick) % (menu_contr_player_2_method - menu_contr_player_1_method) + 1;
	    if (which_action == 2)
		result = set_action_joy_button(which_player, fire, get_button(which_player));
	    else
		result = set_action_joy_button(which_player, fire_blast, get_button(which_player));
#if DEBUG >= EXTRA_DEBUG
	    fprintf(stderr, "set_action_joy_button(%d,X,X) failed with code: %d\n", which_player, result);
#endif /* DEBUG */
	    clean_menu_item(*to, which_action);
	    delete_menu(menu_items);
	    menu_items = create_menu((const char **)create_controls_menu(which_player),
		    (const enum menu_ids *)create_controls_menu_id(which_player));
	    return LOOP_REDRAW_ACTIVE;
	case menu_contr_player_1_joy_cal:
	case menu_contr_player_2_joy_cal:
	    which_player = (action_id - menu_contr_player_1_joystick) / (menu_contr_player_2_method - menu_contr_player_1_method);
	    calibrate_joystick(*to, which_player);
	    return LOOP_RESTART;
	case menu_players:
	    if (button == SDL_BUTTON_LEFT) {
		players++;
		if (players > maximum_ships_count)
		    players = 1;
	    } else {
		if (button == SDL_BUTTON_RIGHT) {
		    players--;
		    if (players == 0)
			players = maximum_ships_count;
		} else
		    break;
	    }
	    delete_menu(menu_items);
	    menu_items = create_menu((const char **)create_options_menu(), options_menu_id);
	    return LOOP_REDRAW_ACTIVE;
	case menu_fullscreen:
	    try_fullscreen = !try_fullscreen;
	    SDL_WM_ToggleFullScreen(*to);
	    return LOOP_CONT;
	case menu_return:
	    delete_menu(menu_items);
	    menu_items = create_menu(menu, menu_id);
	    return LOOP_RESTART;
	case menu_screen_size:
	    screen_resolutions = SDL_ListModes(NULL, SDL_FULLSCREEN | SDL_SWSURFACE | SDL_ANYFORMAT);
	    assert(screen_resolutions);
	    counter = count = 0;
	    while (screen_resolutions[count])
		count++;
	    while (counter < count && (screen_resolutions[counter]->w != screen_width || screen_resolutions[counter]->h != screen_height))
		counter++;
	    if (button == SDL_BUTTON_LEFT) {
		if (--counter < 0)
		    counter = count - 1;
	    } else {
		if (++counter >= count)
		    counter = 0;
	    }
#if DEBUG >= EXTRA_DEBUG
	    printf("Switch to resolution: %ux%u.\n", screen_resolutions[counter]->w, screen_resolutions[counter]->h);
#endif /* DEBUG */
	    if (open_screen(screen_resolutions[counter]->w, screen_resolutions[counter]->h) < 0)
		exit(1);
	    delete_menu(menu_items);
	    menu_items = create_menu((const char **)create_options_menu(), options_menu_id);
	    *to = screen;
	    return LOOP_RESTART;
	default:
#if DEBUG >= EXTRA_DEBUG
	    printf("Unhandled action id: %d!\n", action_id);
#endif /* DEBUG */
	    break;
    }
    return LOOP_CONT;
}

void
draw_text_centered(SDL_Surface *to, const char *text, const int ptsize, const Uint16 y_pos, const SDL_Color fg, const SDL_Color bg, SDL_Rect *clean_rect)
{
    TTF_Font *this_font;
    SDL_Surface *render;
    
    this_font = TTF_OpenFont(get_file(DEFAULT_FONT_NAME, font_file), ptsize);
    render = TTF_RenderText_Shaded(this_font, text, fg, bg);
    SDL_SetColorKey(render, SDL_SRCCOLORKEY, SDL_MapRGB(render->format, bg.r, bg.g, bg.b));
    clean_rect->y = y_pos - render->h / 2;
    if (clean_rect->y < 0)
	clean_rect->y = 0;
    clean_rect->x = (screen_width - render->w) / 2;
    clean_rect->w = render->w;
    clean_rect->h = render->h;
    SDL_BlitSurface(render, NULL, to, clean_rect);
    SDL_UpdateRects(to, 1, clean_rect);
    SDL_FreeSurface(render);
    TTF_CloseFont(this_font);
}
