#include <stdio.h>
#include "config.h"

#ifdef STDC_HEADERS
#   include <stdlib.h>
#   include <string.h>
#else /* !STDC_HEADERS */
#   ifndef HAVE_MEMCPY
#	define memcpy(d, s, n)	bcopy((s), (d), (n))
#	define memmove(d, s, n)	bcopy((s), (d), (n))
#   endif /* !HAVE_MEMCPY */
#endif /* !STDC_HEADERS */

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#   include <sys/types.h>
#   include <sys/stat.h>
#endif /* HAVE_UNISTD_H */

#ifdef HAVE_MEMORY_H
#   include <memory.h>
#endif /* HAVE_MEMORY_H */

#ifdef HAVE_LIBSDL_IMAGE
#include <SDL/SDL_image.h>
#endif /* HAVE_LIBSDL_IMAGE */

#include "common.h"
#include "files.h"
#include "debug.h"

char filename_buffer[MAX_FILE_NAME_LENGHT + 1];
/*char filename_suffix_buffer[IMAGE_SUFFIXES_COUNT][MAX_RELATIVE_PATH_LENGHT + 1];*/
suffix_str_array_type filename_suffix_buffer;

const suffix_type suffixes[IMAGE_SUFFIXES_COUNT] = {
    { "png", 1 },
    { "jpg", 0 },
    { "bmp", 0 }
};

const int bmp_suffix = 2;
const SDL_Color gauge_start = { 0xFF, 0x0, 0x0, 0 }, gauge_end = { 0x0, 0xFF, 0x0, 0 };

SDL_Surface *
try_load_image(const char *filename_part, const int ship_number, const Uint8 transparent)
{
    int counter;
    char /**files[IMAGE_SUFFIXES_COUNT],*/ *file;
    SDL_Surface *image;
/*    SDL_Color *color;*/
    Uint8 index8;
    Uint16 index16;
    Uint32 index32;
    
    image = NULL;
    if (ship_number >= 0) {
	try_subst_suffixes(filename_part, ship_number);
	for (counter = 0; counter < IMAGE_SUFFIXES_COUNT; counter++)
	    if ((file = get_file((char *)filename_suffix_buffer[counter], image_file)) != NULL) {
		if (counter == bmp_suffix) {
		    image = SDL_LoadBMP(file);
		}
#ifdef HAVE_LIBSDL_IMAGE
		else
		    image = IMG_Load(file);
#else /* !HAVE_LIBSDL_IMAGE */
		else
		    fprintf(stderr, "Warning: To support this type of image file you need libsdl-image library or\nuse bmp image format instead.\n");
#endif /* !HAVE_LIBSDL_IMAGE */
		if (image) {
		    if (transparent == 1 && suffixes[counter].transparent_support == 0) {
			switch (image->format->BitsPerPixel) {
			    case 8:
				index8=*(Uint8 *)image->pixels;
/*				color=fmt->palette->colors[index8];*/
				SDL_SetColorKey(image, SDL_SRCCOLORKEY, (Uint32)index8);
				break;
			    case 16:
				index16=*(Uint16 *)image->pixels;
				SDL_SetColorKey(image, SDL_SRCCOLORKEY, (Uint32)index16);
				break;
			    case 24:
				index32 = *(Uint32 *)image->pixels & 0xFFFFFF;
				SDL_SetColorKey(image, SDL_SRCCOLORKEY, index32);
				break;
			    case 32:
				index32=*(Uint32 *)image->pixels;
				SDL_SetColorKey(image, SDL_SRCCOLORKEY, index32);
				break;
			    default:
#if DEBUG >= NORMAL_DEBUG
				fprintf(stderr, "Not supported bpp for surface %d!\n", image->format->BitsPerPixel);
#endif /* DEBUG */
			}
		    }
		    return image;
		}
#if DEBUG >= NORMAL_DEBUG
		else
		    fprintf(stderr, "Error when loading %s image!\n", file);
#endif /* DEBUG */
	    }
    }
    try_subst_suffixes(filename_part, -1);
    for (counter = 0; counter < IMAGE_SUFFIXES_COUNT; counter++)
	if ((file = get_file(filename_suffix_buffer[counter], image_file)) != NULL) {
	    if (counter == bmp_suffix) {
		image = SDL_LoadBMP(file);
	    }
	    else
#ifdef HAVE_LIBSDL_IMAGE
		image = IMG_Load(file);
#endif /* HAVE_LIBSDL_IMAGE */
	    if (image) {
		    if (transparent == 1 && suffixes[counter].transparent_support == 0) {
			switch (image->format->BitsPerPixel) {
			    case 8:
				index8=*(Uint8 *)image->pixels;
/*				color=fmt->palette->colors[index8];*/
				SDL_SetColorKey(image, SDL_SRCCOLORKEY, (Uint32)index8);
				break;
			    case 16:
				index16=*(Uint16 *)image->pixels;
				SDL_SetColorKey(image, SDL_SRCCOLORKEY, (Uint32)index16);
				break;
			    case 24:
				index32 = *(Uint32 *)image->pixels & 0xFFFFFF;
				SDL_SetColorKey(image, SDL_SRCCOLORKEY, (Uint32)index32);
				break;
			    case 32:
				index32=*(Uint32 *)image->pixels;
				SDL_SetColorKey(image, SDL_SRCCOLORKEY, index32);
				break;
			    default:
#if DEBUG >= NORMAL_DEBUG
				fprintf(stderr, "Not supported bpp for surface %d!\n", image->format->BitsPerPixel);
#endif /* DEBUG */
			}
		    }
		return image;
	    }
#if DEBUG >= NORMAL_DEBUG
	    else
		fprintf(stderr, "Error when loading %s image!\n", file);
#endif /* DEBUG */
	}
    if (!image)
	fprintf(stderr, "Can't find image for %s!\n", filename_part);
    return NULL;
}

suffix_str_array_type *
try_subst_suffixes(const char *fmt, const int ship_number)
{
    int counter;
    char new_fmt[MAX_RELATIVE_PATH_LENGHT + 1];

    snprintf(new_fmt, MAX_RELATIVE_PATH_LENGHT + 1, fmt, (ship_number < 0 ? ".%s" : "%d.%s"));
    if (ship_number < 0)
	for (counter = 0; counter < IMAGE_SUFFIXES_COUNT; counter++)
	    snprintf(filename_suffix_buffer[counter], MAX_RELATIVE_PATH_LENGHT + 1, new_fmt, suffixes[counter].suffix_str);
    else
	for (counter = 0; counter < IMAGE_SUFFIXES_COUNT; counter++)
	    snprintf(filename_suffix_buffer[counter], MAX_RELATIVE_PATH_LENGHT + 1, new_fmt, ship_number, suffixes[counter].suffix_str);
    return &filename_suffix_buffer;
}

char *
get_file(const char *filename, const file_type file)
{
    char *tmp;
    struct stat info;

    getcwd(filename_buffer, MAX_FILE_NAME_LENGHT);
    strcat(filename_buffer, "/");
    switch (file) {
	case other_file:
	    strcat(filename_buffer, OTHER_SUBDIRECTORY);
	    break;
	case config_file:
	    strcat(filename_buffer, CONFIGS_SUBDIRECTORY);
	    break;
	case image_file:
	    strcat(filename_buffer, IMAGES_SUBDIRECTORY);
	    break;
#ifdef HAVE_LIBSDL_TTF
	case font_file:
	    strcat(filename_buffer, FONTS_SUBDIRECTORY);
	    break;
#endif /* HAVE_LIBSDL_TTF */
    }
    strcat(filename_buffer, filename);
#if DEBUG > NORMAL_DEBUG
    printf("Trying `%s'.\n", filename_buffer);
#endif /* DEBUG */
    if (stat(filename_buffer, &info) == 0)
	return filename_buffer;
    tmp = getenv("HOME");
    if (tmp) {
	sprintf(filename_buffer, "%s/."PACKAGE"/", tmp);
	switch (file) {
	    case other_file:
		strcat(filename_buffer, OTHER_SUBDIRECTORY);
		break;
	    case config_file:
		strcat(filename_buffer, CONFIGS_SUBDIRECTORY);
		break;
	    case image_file:
		strcat(filename_buffer, IMAGES_SUBDIRECTORY);
		break;
#ifdef HAVE_LIBSDL_TTF
	    case font_file:
		strcat(filename_buffer, FONTS_SUBDIRECTORY);
		break;
#endif /* HAVE_LIBSDL_TTF */
	}
	strcat(filename_buffer, filename);
#if DEBUG > NORMAL_DEBUG
	printf("Trying `%s'.\n", filename_buffer);
#endif /* DEBUG */
	if (stat(filename_buffer, &info) == 0)
	    return filename_buffer;
    }
    sprintf(filename_buffer, DATADIR"/share/"PACKAGE"-"VERSION"/");
    switch (file) {
	case other_file:
	    strcat(filename_buffer, OTHER_SUBDIRECTORY);
	    break;
	case config_file:
	    strcat(filename_buffer, CONFIGS_SUBDIRECTORY);
	    break;
	case image_file:
	    strcat(filename_buffer, IMAGES_SUBDIRECTORY);
	    break;
#ifdef HAVE_LIBSDL_TTF
	case font_file:
	    strcat(filename_buffer, FONTS_SUBDIRECTORY);
	    break;
#endif /* HAVE_LIBSDL_TTF */
    }
    strcat(filename_buffer, filename);
#if DEBUG > NORMAL_DEBUG
    printf("Trying `%s'.\n", filename_buffer);
#endif /* DEBUG */
    if (stat(filename_buffer, &info) == 0)
	return filename_buffer;
    sprintf(filename_buffer, DATADIR"/share/"PACKAGE"/");
    switch (file) {
	case other_file:
	    strcat(filename_buffer, OTHER_SUBDIRECTORY);
	    break;
	case config_file:
	    strcat(filename_buffer, CONFIGS_SUBDIRECTORY);
	    break;
	case image_file:
	    strcat(filename_buffer, IMAGES_SUBDIRECTORY);
	    break;
#ifdef HAVE_LIBSDL_TTF
	case font_file:
	    strcat(filename_buffer, FONTS_SUBDIRECTORY);
	    break;
#endif /* HAVE_LIBSDL_TTF */
    }
    strcat(filename_buffer, filename);
#if DEBUG > NORMAL_DEBUG
    printf("Trying `%s'.\n", filename_buffer);
#endif /* DEBUG */
    if (stat(filename_buffer, &info) == 0)
	return filename_buffer;
    return NULL;
}


void
draw_line(SDL_Surface *to, const Uint16 x1, const Uint16 y1, const Uint16 x2, const Uint16 y2, const SDL_Color *color)
{
    Uint16 my_x1, my_x2, my_y1, my_y2;
    Uint8 color8;
    Uint16 color16;
    Uint32 color32, color24, mask24 = 0, *changed;
    int counter;
    
    /* I draw only straight lines */
    if (x1 != x2 && y1 != y2)
	return;
    if (x1 > x2) {
	my_x1 = x2;
	my_x2 = x1;
    } else {
	my_x1 = x1;
	my_x2 = x2;
    }
    if (y1 > y2) {
	my_y1 = y2;
	my_y2 = y1;
    } else {
	my_y1 = y1;
	my_y2 = y2;
    }
    if (my_y2 >= to->h)
	my_y2 = to->h - 1;
    if (my_x2 >= to->w)
	my_x2 = to->w - 1;
    /* Use clipping */
    if (my_x1 < to->clip_rect.x)
	my_x1 = to->clip_rect.x;
    if (my_y1 < to->clip_rect.y)
	my_y1 = to->clip_rect.y;
    if (my_x2 >= to->clip_rect.x + to->clip_rect.w)
	my_x2 = to->clip_rect.x + to->clip_rect.w - 1;
    if (my_y2 >= to->clip_rect.y + to->clip_rect.h)
	my_y2 = to->clip_rect.y + to->clip_rect.h - 1;
    switch (to->format->BitsPerPixel) {
	case 8:
	    color8 = (Uint8)SDL_MapRGB(to->format, color->r, color->g, color->b);
	    if (my_y1 == my_y2)
		memset((void *)(((Uint8 *)to->pixels) + my_x1 + my_y1 * to->w), color8, my_x2 - my_x1);
	    else
		for (counter = my_x1 + my_y1 * to->w; counter < (my_y2 + 1) * to->w; counter += to->w)
		    ((Uint8 *)to->pixels)[counter] = color8;
	    break;
	case 16:
	    color16 = (Uint16)SDL_MapRGB(to->format, color->r, color->g, color->b);
	    if (my_y1 == my_y2)
		for (counter = my_x1 + my_y1 * to->w; counter <= my_x2 + my_y1 * to->w; counter++)
		    ((Uint16 *)to->pixels)[counter] = color16;
	    else
		for (counter = my_x1 + my_y1 * to->w; counter < (my_y2 + 1) * to->w; counter += to->w)
		    ((Uint16 *)to->pixels)[counter] = color16;
	    break;
	case 24:
	    color24 = (Uint32)SDL_MapRGB(to->format, color->r, color->g, color->b);
	    ((Uint8 *)&mask24)[3] = 0xFF;
	    if (my_y1 == my_y2)
		for (counter = my_x1 + my_y1 * to->w * to->format->BytesPerPixel; counter <= my_x2 + my_y1 * to->w * to->format->BytesPerPixel; counter += to->format->BytesPerPixel) {
		    changed = (Uint32 *)((Uint8 *)to->pixels + counter);
		    *changed &= mask24;
		    *changed |= color24;
		}
	    else
		for (counter = my_x1 + my_y1 * to->w * to->format->BytesPerPixel; counter < (my_y2 + 1) * to->w * to->format->BytesPerPixel; counter += to->w * to->format->BytesPerPixel) {
		    changed = (Uint32 *)((Uint8 *)to->pixels + counter);
		    *changed &= mask24;
		    *changed |= color24;
		}
	    break;
	case 32:
	    color32 = SDL_MapRGB(to->format, color->r, color->g, color->b);
	    if (my_y1 == my_y2)
		for (counter = my_x1 + my_y1 * to->w; counter <= my_x2 + my_y1 * to->w; counter++)
		    ((Uint32 *)to->pixels)[counter] = color32;
	    else
		for (counter = my_x1 + my_y1 * to->w; counter < (my_y2 + 1) * to->w; counter += to->w)
		    ((Uint32 *)to->pixels)[counter] = color32;
	    break;
    }
}

void
draw_rect(SDL_Surface *to, const SDL_Rect *rect, const SDL_Color *color)
{
    draw_line(to, rect->x, rect->y, rect->x + rect->w - 1, rect->y, color);
    draw_line(to, rect->x + rect->w - 1, rect->y, rect->x + rect->w - 1, rect->y + rect->h - 1, color);
    draw_line(to, rect->x, rect->y + rect->h - 1, rect->x + rect->w - 1, rect->y + rect->h - 1, color);
    draw_line(to, rect->x, rect->y, rect->x, rect->y + rect->h - 1, color);
}

void
draw_cross(SDL_Surface *to, const Uint16 x, const Uint16 y, const Uint16 size, const SDL_Color *color) {
    draw_line(to, x + 1 - size / 2 - size % 2, y, x + size / 2, y, color);
    draw_line(to, x, y + 1 - size / 2 - size % 2, x, y + size / 2, color);
}

void
draw_gauge(SDL_Surface *to, const SDL_Rect *rect, const float percent)
{
    SDL_Surface *surface;
    SDL_Rect my_rect;
    Uint16 my_x1, my_y1, my_x2, my_y2, x_counter, y_counter;
    Sint16 delta_r, delta_g, delta_b;
    Uint8 color8;
    Uint16 color16;
    Uint32 color32, color24, mask24 = 0, *changed;
    SDL_Color white = { 0xFF, 0xFF, 0xFF, 0 };

    if (rect->w <= 5 || rect->h < 5)
	return;
    surface = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCCOLORKEY | SDL_SRCALPHA, rect->w, rect->h,
	    to->format->BitsPerPixel, to->format->Rmask, to->format->Gmask, to->format->Bmask, to->format->Amask);
    my_rect.x = 0;
    my_rect.y = 0;
    my_rect.w = rect->w;
    my_rect.h = rect->h;
    SDL_FillRect(surface, &my_rect, SDL_MapRGB(surface->format, 0, 0, 0));
    SDL_SetColorKey(surface, SDL_SRCCOLORKEY, SDL_MapRGB(surface->format, 0, 0, 0));
    SDL_SetAlpha(surface, SDL_SRCALPHA, SET_0_TRANSPARENT_100_OPAQUE(70));
    draw_rect(surface, &my_rect, &white);
    my_x1 = 2;
    my_x2 = rect->w - 3;
    my_y1 = 2;
    my_y2 = rect->h - 3;
    delta_r = gauge_end.r - gauge_start.r;
    delta_g = gauge_end.g - gauge_start.g;
    delta_b = gauge_end.b - gauge_start.b;
    switch (surface->format->BitsPerPixel) {
	case 8:
	    for (x_counter = my_x1; x_counter <= my_x2; x_counter++) {
		if (100 * x_counter / (float)my_x2 > percent)
		    break;
		color8 = (Uint8)SDL_MapRGB(surface->format,
			gauge_start.r + (delta_r * x_counter / my_x2),
			gauge_start.g + (delta_g * x_counter / my_x2),
			gauge_start.b + (delta_b * x_counter / my_x2));
		for (y_counter = my_y1; y_counter <= my_y2; y_counter++)
		    ((Uint8 *)surface->pixels)[y_counter * surface->w + x_counter] = color8;
	    }
	    break;
	case 16:
	    for (x_counter = my_x1; x_counter <= my_x2; x_counter++) {
		if (100 * x_counter / (float)my_x2 > percent)
		    break;
		color16 = (Uint16)SDL_MapRGB(surface->format,
			gauge_start.r + (delta_r * x_counter / my_x2),
			gauge_start.g + (delta_g * x_counter / my_x2),
			gauge_start.b + (delta_b * x_counter / my_x2));
		for (y_counter = my_y1; y_counter <= my_y2; y_counter++)
		    ((Uint16 *)surface->pixels)[y_counter * surface->w + x_counter] = color16;
	    }
	    break;
	case 24:
	    ((Uint8 *)&mask24)[3] = 0xFF;
	    for (x_counter = my_x1; x_counter <= my_x2; x_counter++) {
		if (100 * x_counter / (float)my_x2 > percent)
		    break;
		color24 = (Uint32)SDL_MapRGB(surface->format,
			gauge_start.r + (delta_r * x_counter / my_x2),
			gauge_start.g + (delta_g * x_counter / my_x2),
			gauge_start.b + (delta_b * x_counter / my_x2));
		for (y_counter = my_y1; y_counter <= my_y2; y_counter++) {
		    changed = (Uint32 *)((Uint8 *)surface->pixels + x_counter +
			    y_counter * surface->w * surface->format->BytesPerPixel);
		    *changed &= mask24;
		    *changed |= color24;
		}
	    }
	    break;
	case 32:
	    for (x_counter = my_x1; x_counter <= my_x2; x_counter++) {
		if (100 * x_counter / (float)my_x2 > percent)
		    break;
		color32 = (Uint32)SDL_MapRGB(surface->format,
			gauge_start.r + (delta_r * x_counter / my_x2),
			gauge_start.g + (delta_g * x_counter / my_x2),
			gauge_start.b + (delta_b * x_counter / my_x2));
		for (y_counter = my_y1; y_counter <= my_y2; y_counter++)
		    ((Uint32 *)surface->pixels)[y_counter * surface->w + x_counter] = color32;
	    }
	    break;
    }
    my_rect.x = rect->x;
    my_rect.y = rect->y;
    my_rect.w = rect->w;
    my_rect.h = rect->h;
    SDL_BlitSurface(surface, NULL, to, &my_rect);
    SDL_FreeSurface(surface);
}
