#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <SDL/SDL.h>
#include "config.h"
#include "fonts.h"
#include "files.h"
#include "common.h"

#ifdef HAVE_LIBSDL_TTF

TTF_Font **fonts;
int fonts_count;

void
init_fonts(void)
{
    if (TTF_Init() < 0) {
	fprintf(stderr, "Couldn't initialize TTF: %s\n", SDL_GetError());
	exit(EXIT_FAILURE);
    }
    fonts = NULL;
    fonts_count = 0;
}

int
open_font(const int font_size)
{
    TTF_Font *new_font;
    
    new_font = TTF_OpenFont(get_file(DEFAULT_FONT_NAME, font_file), font_size);
    if (!new_font) {
	fprintf(stderr, "Couldn't load %d pt font from %s: %s\n", font_size,
		get_file(DEFAULT_FONT_NAME, font_file), SDL_GetError());
	return -1;
    }
    if (fonts)
	fonts = (TTF_Font **)realloc((void *)fonts, sizeof(TTF_Font *) * ++fonts_count);
    else {
	fonts = (TTF_Font **)malloc(sizeof(TTF_Font *));
	fonts_count = 1;
    }
    if (!fonts)
	perror("fonts.c:");
    fonts[fonts_count - 1] = new_font;
    return (fonts_count - 1);
}

void
free_fonts(void)
{
    int counter;

    for (counter = 0; counter < fonts_count; counter++) {
	TTF_CloseFont(fonts[counter]);
    }
    free(fonts);
    fonts = NULL;
    TTF_Quit();
}

TTF_Font *
get_font(const int font_number)
{
    assert(font_number < fonts_count);
    return fonts[font_number];
}

#endif /* HAVE_LIBSDL_TTF */
