#ifndef INPUT_H
#define INPUT_H

#include <SDL/SDL.h>
#include "config.h"

#define ACTIONS_COUNT		6

#define QUIT_EVENT		1

#define NO_ERROR			0
#define ERR_NO_RIGHT_INPUT_SELECTED	-1
#define ERR_BAD_BUTTON_NUMBER		-2
#define ERR_NOT_IMPLEMENTED		-3
#define ERR_BAD_JOYSTICK_NUMBER		-4
#define ERR_SDL_ERROR			-5

#define ALL_DONE			0
#define ONE_MORE			1

#define QUIT_KEY		SDLK_ESCAPE
#define ACTION_INDEX(x)		((x)-1)

typedef enum {
    speed_up = 1,
    speed_down,
    rotate_left,
    rotate_right,
    fire,
    fire_blast
} player_actions;

typedef enum {
    no_input = 0,
    keyboard,
    joystick
} input_method;

typedef struct {
    input_method type;
    SDLKey keys[ACTIONS_COUNT];
} keyboard_type;

#ifdef ENABLE_JOYSTICK
#define MAX_JOYS_COUNT	    64
#define AXIS_MIN_DEFAULTS	-3200
#define AXIS_MAX_DEFAULTS	3200

SDL_Joystick *joysticks[MAX_JOYS_COUNT];

typedef struct {
    Sint16 y_down, y_up, x_left, x_right;
} joy_params_type;

joy_params_type joysticks_params[MAX_JOYS_COUNT];

typedef struct {
    input_method type;
    Uint8 joy_active, joy_number, fire_button, blast_button;
} joystick_type;
#endif /* ENABLE_JOYSTICK */

typedef union {
    input_method type;
    keyboard_type keyboard;
#ifdef ENABLE_JOYSTICK
    joystick_type joystick;
#endif /* ENABLE_JOYSTICK */
} input_type;

void init_input(const int max_players);
void close_input(void);
int is_action_active(const int which_player, const player_actions action);
int get_system_requests(void);
int set_action_key(const int which_player, const player_actions action, const SDLKey key);
SDLKey get_action_key(const int which_player, const player_actions action);
#ifdef ENABLE_JOYSTICK
int is_joystick_active(const int which_player);
int open_joystick(const Uint8 which);
int set_action_joy_button(const int which_player, const player_actions action, const Uint8 button);
int set_joystick(const int which_player, const Uint8 joystick_number);
Uint8 get_joystick(const int which_player);
Uint8 get_action_joy_button(const int which_player, const player_actions action);
void calibrate_joystick(SDL_Surface *to, const int which_player);
Uint8 get_button(const int which_player);
#endif /* ENABLE_JOYSTICK */
void set_players_input_method(const int which_player, const input_method method);
input_method get_players_input_method(const int which_player);
char *get_input_method_desc(const input_method method);
SDLKey get_key(void);
int work_events(void);

#endif /* INPUT_H */
