#include <stdio.h>
#include "config.h"

#ifdef STDC_HEADERS
#   include <stdlib.h>
#   include <string.h>
#else /* !STDC_HEADERS */
#   ifndef HAVE_MEMCPY
#	define memcpy(d, s, n)	bcopy((s), (d), (n))
#	define memmove(d, s, n)	bcopy((s), (d), (n))
#   endif /* !HAVE_MEMCPY */
#endif /* !STDC_HEADERS */

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#   include <sys/types.h>
#   include <sys/stat.h>
#endif /* HAVE_UNISTD_H */

#include <time.h>
#include <SDL/SDL.h>
#include "main.h"
#include "background.h"
#include "ship.h"
#include "explosion.h"
#include "terminal.h"
#include "input.h"
#include "shot.h"
#include "files.h"
#include "common.h"
#include "debug.h"
#include "options.h"
#include "map.h"
#include "gauges.h"
#ifdef HAVE_LIBSDL_TTF
#   include "fonts.h"
/*#   include "menu.h"*/
#   include "ask.h"
#endif /* HAVE_LIBSDL_TTF */

#ifdef DO_TESTS
#   include "test.h"
#endif /* DO_TESTS */

int maximum_ships_count = DEFAULT_MAXIMUM_SHIPS_COUNT, current_ships_count;

SDL_Surface *screen = NULL;
int quit, cancel;
settings_type *system_settings, *repair_settings;

/*
 * open_screen	- opens screen and fills screen variables with valid content
 *
 * IN:
 *  width	wanted screen width
 *  height	wanted screen height
 * RETURN VALUE:
 *  0	OK
 *  -1	FAILURE
 */
int
open_screen(const int width, const int height)
{
    Uint32 bpp, flags;
    
    if (screen)
	SDL_FreeSurface(screen);
    flags = SDL_SWSURFACE | SDL_ANYFORMAT;
    if (try_fullscreen)
	flags |= SDL_FULLSCREEN;
    bpp = SDL_VideoModeOK(width, height, 16, flags);
    if (!bpp) {
#if DEBUG >= NORMAL_DEBUG
	fprintf(stderr, "open_screen: Requested mode (%ux%u) is not available!\n", width, height);
#endif /* DEBUG */
	flags = SDL_SWSURFACE | SDL_ANYFORMAT;
	screen = SDL_SetVideoMode(640, 480, 8, flags);
	if (!screen) {
	    fprintf(stderr, "open_screen: Can't open screen at fallback resolution 640x480: %s!\n", SDL_GetError());
	    return -1;
	}
	screen_width = 640;
	screen_height = 480;

	return 0;
    }
    screen = SDL_SetVideoMode(width, height, bpp, flags);
    if (!screen) {
	fprintf(stderr, "open_screen: Can't open screen at mode %ux%u@%u: %s!\n", width, height, bpp, SDL_GetError());
	return -1;
    }
    screen_width = width;
    screen_height = height;

    return 0;
}

void
ShowLogo()
{
    SDL_Surface *logo;
    SDL_Rect logo_rect;
    int counter;
    clock_t start, end;
    Uint32 black;

    black = SDL_MapRGB(screen->format, 0, 0, 0);
    logo = try_load_image(LOGO_FILE_NAME, -1, 1);
    logo_rect.w = logo->w;
    logo_rect.h = logo->h;
    logo_rect.x = (screen_width - logo_rect.w) / 2;
    logo_rect.y = (screen_height - logo_rect.h) / 2;
    for (counter = 0; counter < LOGO_COUNT; counter++) {
	start = clock();
	if (SDL_QuitRequested())
	    exit(EXIT_SUCCESS);
	SDL_FillRect(screen, &logo_rect, black);
	SDL_SetAlpha(logo, SDL_SRCALPHA, SET_0_TRANSPARENT_100_OPAQUE(100 * counter / LOGO_COUNT));
	SDL_BlitSurface(logo, NULL, screen, &logo_rect);
	SDL_UpdateRects(screen, 1, &logo_rect);
	end = clock();
	if ((end - start) < LOGO_DELAY) {
	    SDL_Delay((LOGO_DELAY - (end - start)) / CLOCKS_PER_MILISEC);
	}
    }
}

void
UpdateShip(const int ship_number)
{
    if (is_destroyed_ship(ship_number))
	return;
    if (is_action_active(ship_number, speed_up))
	increase_ship_speed(ship_number);
    if (is_action_active(ship_number, speed_down))
	decrease_ship_speed(ship_number);
    if (is_action_active(ship_number, rotate_left))
	rotate_ship(ship_number, rotate_by_ship(ship_number));
    if (is_action_active(ship_number, rotate_right))
	rotate_ship(ship_number, -rotate_by_ship(ship_number));
    if (is_action_active(ship_number, fire))
	create_and_init_shot(get_ship(ship_number), normal_shot);
    if (is_action_active(ship_number, fire_blast))
	create_and_init_shot(get_ship(ship_number), blast_shot);
}

int
Run()
{
    clock_t start, end;
    int ship, counter, count;

    draw_background(screen, NULL);
    SDL_Flip(screen);
    cancel = 0;
    counter = 0;
    while (!cancel) {
	start = clock();
	cancel = (get_system_requests() & QUIT_EVENT ? 1 : 0);
	if (counter++ > SEC(REPAIR_DELAY)) {
	    repair_all_ships();
	    counter = 0;
	}
	count = 0;
	for (ship = 0; ship < maximum_ships_count; ship++)
	    if (is_created_ship(ship) && (!is_destroyed_ship(ship)))
		UpdateShip(ship);
		count++;
	prepare_draw_gauges(screen);
	do_move_with_all_ships(screen);
	do_move_with_all_shots(screen);
	update_all_ships(screen);
	update_all_shots(screen);
	continue_exploding(screen);
	draw_all_gauges(screen);
	update_all_gauges(screen);
	end = clock();
	if ((end - start) < (CLOCKS_PER_SEC / FPS)) {
	    SDL_Delay(((CLOCKS_PER_SEC / FPS) - (end - start)) / CLOCKS_PER_MILISEC);
	}
	if (!count)
	    break;
    }
    return 0;
}

void
OnQuit(void)
{
    if (current_ships_count) {
	free(system_settings);
	free(repair_settings);
    }
/*    write_keys_to_file();*/
    done_options();
    free_shots_array();
    free_ships();
    free_explosions();
#ifdef HAVE_LIBSDL_TTF
    free_menu();
    free_fonts();
#endif /* HAVE_LIBSDL_TTF */
    free_map();
    free_background();
    close_input();
    SDL_FreeSurface(screen);
}

int
main(int argc, char *argv[])
{
    int counter;

    if (SDL_Init(SDL_INIT_VIDEO
#ifdef ENABLE_JOYSTICK
		| SDL_INIT_JOYSTICK
#endif /* ENABLE_JOYSTICK */
		) < 0) {
	fprintf(stderr, "Couldn't initialize SDL: %s\n", SDL_GetError());
        exit(1);
    }
    system_settings = NULL;
    repair_settings = NULL;
    current_ships_count = 0;
    atexit(SDL_Quit);
    atexit(OnQuit);
    quit = 0;
    
    init_options();
    if (open_screen(screen_width, screen_height) < 0)
	exit(1);

    SDL_WM_SetCaption(PACKAGE" "VERSION, PACKAGE);
    srand((unsigned int)time(NULL));

    init_background();
    init_map();
    init_ships(maximum_ships_count);
    init_input(maximum_ships_count);
    init_explosions();
    init_shots_array();
#ifdef HAVE_LIBSDL_TTF
    init_fonts();
    init_menu();
#endif /* HAVE_LIBSDL_TTF */
    read_keys_from_file();

#ifdef DO_TESTS
    test_draw_background(screen, 1000);
    test_map(100000);
    printf("Press any key to exit ...\n");
    getc(stdin);
#else /* !DO_TESTS */
    ShowLogo();
    while (!quit) {
#ifdef HAVE_LIBSDL_TTF
/*	draw_root_menu(screen); */
	draw_menu(screen);
	if (start_menu(screen)) {
	    for (counter = 0; counter < maximum_ships_count; counter++)
		if (is_created_ship(counter))
		    delete_ship(counter);
	    delete_all_shots();
	    delete_all_explosions(screen);
	} else
	    return EXIT_SUCCESS;
#else /* !HAVE_LIBSDL_TTF */
	set_game_settings();
#endif /* !HAVE_LIBSDL_TTF */

	draw_background(screen, NULL);
	Run();

	for (counter = 0; counter < maximum_ships_count; counter++)
	    if (is_created_ship(counter))
		delete_ship(counter);

	if (!continue_question())
	    exit(EXIT_SUCCESS);
	delete_all_shots();
	delete_all_explosions(screen);
    }
#endif /* !DO_TESTS */

    return EXIT_SUCCESS;
}
