#include <assert.h>
#include <stdlib.h>
#include <math.h>
#include "options.h"
#include "map.h"

const size_t hash_size = 1024;

Uint16 x_divide, y_divide, max_x, y_digits;
struct node_type **map = NULL;

/*
 * init_map	- inits map and calculate variables for hash counting
 *
 * IN:
 *  no parameters
 * RETURN VALUE:
 *  none
 */
void
init_map()
{
    int counter;

    map = (struct node_type **)malloc(sizeof(struct node_type *) * hash_size);
    max_x = sqrt(hash_size);
    x_divide = screen_width / max_x + (screen_width % max_x ? 1 : 0);
    y_divide = screen_height / max_x + (screen_height % max_x ? 1 : 0);
    y_digits = ((Uint16)(log(hash_size) / log(2))) / 2;
    for (counter = 0; counter < (int)hash_size; counter++)
	map[counter] = init_list();
}

/*
 * free_map	- frees map
 *
 * IN:
 *  no parameters
 * RETURN VALUE:
 *  none
 */
void
free_map()
{
    int counter;

    if (map) {
	for (counter = 0; counter < (int)hash_size; counter++)
	    clean_list(map[counter]);
	free(map);
	map = NULL;
    }
}

/*
 * reinit_map	- recalculates variables for hash counting (after screen resolution change)
 *
 * IN:
 *  no parameters
 * RETURN VALUE:
 *  none
 */
void
reinit_map()
{
    x_divide = screen_width / max_x + (screen_width % max_x ? 1 : 0);
    y_divide = screen_height / max_x + (screen_height % max_x ? 1 : 0);
}

/*
 * clean_map	- erases all objects from map
 *
 * IN:
 *  no parameters
 * RETURN VALUE:
 *  none
 */
void
clean_map()
{
    int counter;

    for (counter = 0; counter < (int)hash_size; counter++)
	clean_list(map[counter]);
}

/*
 * insert_object_into_map	- inserts object into the map
 *
 * IN:
 *  position	    object coordinates
 *  object	    object
 * RETURN VALUE:
 *  none
 */
void
insert_object_into_map(const SDL_Rect *position, object_type *object)
{
    Sint16 x_counter, y_counter, x_start, x_stop, y_start, y_stop;

    x_start = position->x - position->x % x_divide;
    y_start = position->y - position->y % y_divide;
    if (x_start < 0)
	x_start = 0;
    if (y_start < 0)
	y_start = 0;
    x_stop = position->x + position->w;
    y_stop = position->y + position->h;
    if (x_stop > screen_width)
	x_stop = screen_width;
    if (y_stop > screen_height)
	y_stop = screen_height;
    for (x_counter = x_start; x_counter < x_stop; x_counter += x_divide)
	for (y_counter = y_start; y_counter < y_stop; y_counter += y_divide)
	    insert_into_list(map[get_hash(x_counter, y_counter)], object);
}

/*
 * delete_object_from_map	- deletes object from the map
 *
 * IN:
 *  position	    object coordinates
 *  object	    object
 * RETURN VALUE:
 *  none
 */
void
delete_object_from_map(const SDL_Rect *position, object_type *object)
{
    Sint16 x_counter, y_counter, x_start, x_stop, y_start, y_stop;

    x_start = position->x - position->x % x_divide;
    y_start = position->y - position->y % y_divide;
    if (x_start < 0)
	x_start = 0;
    if (y_start < 0)
	y_start = 0;
    x_stop = position->x + position->w;
    y_stop = position->y + position->h;
    if (x_stop > screen_width)
	x_stop = screen_width;
    if (y_stop > screen_height)
	y_stop = screen_height;
    for (x_counter = x_start; x_counter < x_stop; x_counter += x_divide)
	for (y_counter = y_start; y_counter < y_stop; y_counter += y_divide)
	    delete_from_list(map[get_hash(x_counter, y_counter)], object);
}

/*
 * move_with_object_in_map	- changes object's position in the map
 *
 * IN:
 *  old_position    old object coordinates
 *  new_position    new object coordinates
 *  object	    object
 * RETURN VALUE:
 *  none
 */
void
move_with_object_in_map(const SDL_Rect *old_position, const SDL_Rect *new_position, object_type *object)
{
    Sint16 x_counter, y_counter, x_old_start, x_old_stop, y_old_start, y_old_stop;
    Sint16 x_new_start, x_new_stop, y_new_start, y_new_stop;

    x_old_start = old_position->x - old_position->x % x_divide;
    y_old_start = old_position->y - old_position->y % y_divide;
    x_new_start = new_position->x - new_position->x % x_divide;
    y_new_start = new_position->y - new_position->y % y_divide;
    if (x_old_start == x_new_start && y_old_start == y_new_start)
	return;
    if (x_old_start < 0)
	x_old_start = 0;
    if (y_old_start < 0)
	y_old_start = 0;
    if (x_new_start < 0)
	x_new_start = 0;
    if (y_new_start < 0)
	y_new_start = 0;
    x_old_stop = old_position->x + old_position->w;
    y_old_stop = old_position->y + old_position->h;
    x_new_stop = new_position->x + new_position->w;
    y_new_stop = new_position->y + new_position->h;
    if (x_old_stop > screen_width)
	x_old_stop = screen_width;
    if (y_old_stop > screen_height)
	y_old_stop = screen_height;
    if (x_new_stop > screen_width)
	x_new_stop = screen_width;
    if (y_new_stop > screen_height)
	y_new_stop = screen_height;
    for (x_counter = x_old_start; x_counter < x_old_stop; x_counter += x_divide)
	for (y_counter = y_old_start; y_counter < y_old_stop; y_counter += y_divide)
	    if (x_counter < x_new_start || x_counter > x_new_stop || y_counter < y_new_start || y_counter > y_new_stop)
		delete_from_list(map[get_hash(x_counter, y_counter)], object);
    for (x_counter = x_new_start; x_counter < x_new_stop; x_counter += x_divide)
	for (y_counter = y_new_start; y_counter < y_new_stop; y_counter += y_divide)
	    if (x_counter < x_old_start || x_counter > x_old_stop || y_counter < y_old_start || y_counter > y_old_stop)
		insert_into_list(map[get_hash(x_counter, y_counter)], object);
}

/*
 * get_first_object_from_map	- return first object from map and init walk_ptr
 *
 * IN:
 *  position	    area
 *  walk_ptr	    address of structure where is our current position and other informations
 * RETURN VALUE:
 *  NULL	    on error or no object found
 *  object	    on success
 */
object_type *
get_first_object_from_map(const SDL_Rect *position, map_walk_type *walk_ptr)
{
    object_type *result;

    if (!position || !walk_ptr)
	return NULL;
    walk_ptr->x_cur = position->x - position->x % x_divide;
    walk_ptr->y_start = position->y - position->y % y_divide;
    if (walk_ptr->x_cur < 0)
	walk_ptr->x_cur = 0;
    if (walk_ptr->y_start < 0)
	walk_ptr->y_start = 0;
    walk_ptr->x_stop = position->x + position->w;
    walk_ptr->y_stop = position->y + position->h;
    if (walk_ptr->x_stop > screen_width)
	walk_ptr->x_stop = screen_width;
    if (walk_ptr->y_stop > screen_height)
	walk_ptr->y_stop = screen_height;
    for (; walk_ptr->x_cur < walk_ptr->x_stop; walk_ptr->x_cur += x_divide)
	for (walk_ptr->y_cur = walk_ptr->y_start; walk_ptr->y_cur < walk_ptr->y_stop; walk_ptr->y_cur += y_divide)
	    if ((result = get_first_object(map[get_hash(walk_ptr->x_cur, walk_ptr->y_cur)], &walk_ptr->node_ptr)))
		return result;
    return NULL;
}

/*
 * get_next_object_from_map	- return next object from map
 *
 * IN:
 *  walk_ptr	    address of structure where is our current position and other informations
 * RETURN VALUE:
 *  NULL	    on error or no object found
 *  object	    on success
 */
object_type *
get_next_object_from_map(map_walk_type *walk_ptr)
{
    object_type *result;

    if (!walk_ptr)
	return NULL;

    for (; walk_ptr->x_cur < walk_ptr->x_stop; walk_ptr->x_cur += x_divide) {
	for (; walk_ptr->y_cur < walk_ptr->y_stop; walk_ptr->y_cur += y_divide)
	    if (walk_ptr->node_ptr) {
		if ((result = get_next_object(&walk_ptr->node_ptr)))
		    return result;
		else {
		    walk_ptr->node_ptr = NULL;
		    continue;
		}
	    } else
		if ((result = get_first_object(map[get_hash(walk_ptr->x_cur, walk_ptr->y_cur)], &walk_ptr->node_ptr)))
		    return result;
	walk_ptr->y_cur = walk_ptr->y_start;
    }
    return NULL;
}

/*
 * get_hash	- returns hash based on coordinates
 *
 * IN:
 *  x		    x position
 *  y		    y position
 * RETURN VALUE:
 *  hash
 */
int
get_hash(const Uint16 x, const Uint16 y)
{
    int result;

    result = y / y_divide;
    result = ((result << y_digits) + x / x_divide) % hash_size;

    return result;
}
