#ifndef OBJECTS_H
#define OBJECTS_H

#include <SDL/SDL.h>

/* need_update values: */
#define DRAW_UPDATE		1
#define CLEAN_UPDATE		2
#define NEED_REDRAW		4

/* options values: */
#define NO_OPTIONS
#define DO_NOT_DRAW		1

#define M_PI			3.14159265358979323846

#define SHOT_TYPE		1
#define SHIP_TYPE		2
#define EXPLOSION_TYPE		3

#define SPEED_CONST		8

typedef enum {
    engine_system,
    weapons_system,
    shields_system,
    controls_system
} which_system;

struct ship_system {
    Sint8 repair_speed, max_energy;
    Sint16 condition;
};

struct ship_systems {
    struct ship_system engine, weapons, shields, controls;
};

typedef struct _ship_type {
    Uint8 type;
    SDL_Surface *ship_image;
    SDL_Rect clean_rect, update_rect;
    Uint16 ship_position_left, ship_position_top, ships_count;
    float ship_distance_left, ship_distance_top, ship_speed, motion_angle;
    Uint8 is_shown, need_update, options;

    /* Ship specific */
    SDL_Surface *ship_shield_image, *normal_shot_image, *blast_shot_image;
    Sint8 ship_armor;
    float ship_weapons, ship_shields, ship_angle;
    struct ship_systems systems;
} ship_type;


typedef struct _shot_type {
    Uint8 type;
    SDL_Surface *shot_image;
    SDL_Rect clean_rect, update_rect;
    Uint16 shot_position_left, shot_position_top, not_used;
    float shot_distance_left, shot_distance_top, shot_speed, shot_angle;
    Uint8 is_shown, need_update, options;

    /* Shot specific */
    Uint8 shot_strenght, shot_life;
} shot_type;


typedef struct _explosion_type {
    Uint8 type;
    SDL_Surface *explosion_image;
    SDL_Rect clean_rect, update_rect;
    Uint16 explosion_position_left, explosion_position_top, images_count;
    float explosion_distance_left, explosion_distance_top, explosion_speed, explosion_angle; /* Not used */
    Uint8 is_shown, need_update, options;
    /* Explosion special */
    Sint16 current_frame;
} explosion_type;

typedef struct _space_object_type {
    Uint8 type;
    SDL_Surface *object_image;
    SDL_Rect clean_rect, update_rect;
    Uint16 object_position_left, object_position_top, images_count;
    float object_distance_left, object_distance_top, object_speed, object_angle;
    Uint8 is_shown, need_update, options;
} space_object_type;

typedef union _object_type {
    Uint8 type;
    space_object_type space_object;
    shot_type shot;
    ship_type ship;
    explosion_type explosion;
} object_type;

enum wall_mode_type {stop_object, go_through_wall, bounce_object};

/* Public methods: */
void init_space_object(space_object_type *object);
void free_space_object(space_object_type *object);
void draw_space_object(space_object_type *object, SDL_Surface *to);
void clean_space_object(space_object_type *object, SDL_Surface *to);
void update_space_object(space_object_type *object, SDL_Surface *to);
void purge_space_object(space_object_type *object, SDL_Surface *to);
void do_move_with_space_object(space_object_type *object, SDL_Surface *to, enum wall_mode_type wall_mode);
void change_space_object_position(space_object_type *object, const Sint16 delta_left, const Sint16 delta_top, SDL_Surface *to, enum wall_mode_type wall_mode);

#endif /* OBJECTS_H */
