#include <stdio.h>
#ifdef HAVE_CONFIG_H
#   include "config.h"
#endif /* HAVE_CONFIG_H */
#ifdef STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif /* !STDC_HEADERS */
#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#   include <sys/types.h>
#   include <sys/stat.h>
#endif /* HAVE_UNISTD_H */
#include <errno.h>
#include "common.h"
#include "debug.h"
#include "files.h"
#include "input.h"
#include "main.h"
#include "ship.h"
#include "options.h"

/* MASK FOR SCANF/PRINTF-like functions */
const char *config_mask = "width=%hd;height=%hd;fullscreen=%hu";
const char *keys_mask = "ship%d:IM=%d,SU=%d,SD=%d,RL=%d,RR=%d,F=%d,FB=%d\n";
const int config_mask_on_succ = 3;

/* Global variables and default values */
Sint16	screen_width = 640, screen_height = 480;
Uint8	try_fullscreen = 0;

void
init_options(void)
{
    char *home, filename_buffer[MAX_FILE_NAME_LENGHT + 1];
    
    home = getenv("HOME");
    strcpy(filename_buffer, home);
    strcat(filename_buffer, "/."PACKAGE);
    if (mkdir(filename_buffer, 0755) < 0 && errno != EEXIST)
	perror("mkdir:");

    read_options_from_file();
}

void
done_options(void)
{
    write_keys_to_file();
    write_options_to_file();
}

void
read_options_from_file(void)
{
    int result;
    FILE *options_file;
    Uint16 fs;

    options_file = fopen(get_file(CONFIG_FILE_NAME, config_file), "rt");
    if (!options_file) {
	if (errno != ENOENT)
	    perror("open");
	return;
    }
    result = fscanf(options_file, config_mask, &screen_width, &screen_height, &fs);
    if (result <= 0)
	perror("fscanf");
    try_fullscreen = fs;
#if DEBUG >= NORMAL_DEBUG
    if (result < config_mask_on_succ)
	fprintf(stderr, "Can't read all data from config file. Is corrupted?\n");
#endif /* DEBUG */
    fclose(options_file);
}

void
write_options_to_file(void)
{
    int result;
    char *home, filename_buffer[MAX_FILE_NAME_LENGHT + 1];
    FILE *options_file;

    home = getenv("HOME");
    strcpy(filename_buffer, home);
    strcat(filename_buffer, "/."PACKAGE);
    strcat(filename_buffer, "/"CONFIG_FILE_NAME);
    options_file = fopen(filename_buffer, "wt");
    if (!options_file) {
	if (errno != ENOENT)
	    perror("open");
	return;
    }
    result = fprintf(options_file, config_mask, screen_width, screen_height, (unsigned short int)try_fullscreen);
    if (result <= 0)
	perror("fprintf");
    fclose(options_file);
}

void
read_keys_from_file(void)
{
    FILE *config;
    int result, ship_number, im, su, sd, rl, rr, f, fb;

    config = fopen(get_file(KEYS_FILE_NAME, config_file), "rt");
    if (!config) {
	if (errno != ENOENT)
	    perror("open");
	return;
    }
    while (!feof(config)) {
	if ((result = fscanf(config, keys_mask, &ship_number, &im, &su, &sd, &rl, &rr, &f, &fb)) <= 0)
	    perror("fscanf");
	else {
	    if (ship_number >= maximum_ships_count && ship_number < 0)
		continue;
	    switch (im) {
		case keyboard:
		    set_players_input_method(ship_number, keyboard);
		    switch (result) {
			case 8:
			    set_action_key(ship_number, fire_blast, (SDLKey)fb);
			case 7:
			    set_action_key(ship_number, fire, (SDLKey)f);
			case 6:
			    set_action_key(ship_number, rotate_right, (SDLKey)rr);
			case 5:
			    set_action_key(ship_number, rotate_left, (SDLKey)rl);
			case 4:
			    set_action_key(ship_number, speed_down, (SDLKey)sd);
			case 3:
			    set_action_key(ship_number, speed_up, (SDLKey)su);
			case 2:
			    break;
		    }
		    break;
#ifdef ENABLE_JOYSTICK
		case joystick:
		    set_players_input_method(ship_number, joystick);
		    set_joystick(ship_number, (Uint8)0);
		    switch (result) {
			case 3:
			    set_joystick(ship_number, (Uint8)su);
			case 8:
			    set_action_joy_button(ship_number, fire_blast, (Uint8)fb);
			case 7:
			    set_action_joy_button(ship_number, fire, (Uint8)f);
			case 6:
			case 5:
			case 4:
			case 2:
			    break;
		    }
		    break;
#endif /* ENABLE_JOYSTICK */
		case no_input:
		    set_players_input_method(ship_number, no_input);
	    }
	}
    }
    fclose(config);
}

void
write_keys_to_file(void)
{
    FILE *config;
    int ship_number;
    size_t max_ships;
    char filename_buffer[MAX_FILE_NAME_LENGHT + 1], *home;

    home = getenv("HOME");
    strcpy(filename_buffer, home);
    strcat(filename_buffer, "/."PACKAGE);
    if (mkdir(filename_buffer, 0755) < 0 && errno != EEXIST)
	perror("mkdir:");
    strcat(filename_buffer, "/"KEYS_FILE_NAME);
    config = fopen(filename_buffer, "wt");
    if (!config) {
	perror("open");
	return;
    }
    max_ships = get_max_ships_count();
    for (ship_number = 0; ship_number < (int)max_ships; ship_number++)
	switch (get_players_input_method(ship_number)) {
	    case keyboard:
		fprintf(config, keys_mask, ship_number, keyboard,
			(int)get_action_key(ship_number, speed_up),
			(int)get_action_key(ship_number, speed_down),
			(int)get_action_key(ship_number, rotate_left),
			(int)get_action_key(ship_number, rotate_right),
			(int)get_action_key(ship_number, fire),
			(int)get_action_key(ship_number, fire_blast));
		break;
#ifdef ENABLE_JOYSTICK
	    case joystick:
		fprintf(config, keys_mask, ship_number, joystick,
			get_joystick(ship_number), 0, 0, 0,
			get_action_joy_button(ship_number, fire),
			get_action_joy_button(ship_number, fire_blast));
		break;
#endif /* ENABLE_JOYSTICK */
	    default:
		fprintf(config, keys_mask, ship_number, no_input, 0, 0, 0, 0, 0, 0);
		break;
	}
    fclose(config);
}
