#include <assert.h>
#include <stdlib.h>
#define __USE_ISOC99	    1
/*#define __NO_MATH_INLINES   1*/
#include <math.h>
#include <string.h>
#include <SDL/SDL_image.h>

#include "config.h"
#include "debug.h"
#if DEBUG > NO_DEBUG
#   include <stdio.h>
#endif /* DEBUG */
#include "files.h"
#include "ship.h"
#include "explosion.h"
#include "background.h"
#include "common.h"

#define FROM_0_TO_100_ACCELERATE	20
#define ROTATE_CONST			15
#define WEAPONS_CHARGE_CONST		40
#define SHIELDS_CHARGE_CONST		60
#define COLLISION_STRENGTH_CONST	1	/* Higher number == softer collision */

/*#define get_file(a)			a*/
/*#define hypot(x,y)			sqrt((x) * (x) + (y) * (y))*/
#define set_ship_system(system, condition_value, repair_speed_value, max_energy_value)	\
    system.condition = condition_value; system.repair_speed = repair_speed_value; system.max_energy = max_energy_value;

extern Sint16 screen_width, screen_height;

object_type **ships_array;
size_t ships_count;

/* Functions, that are needed for this block of functions */
float
get_angle_of_vectors_add(const float angle1, const float size1,
			    const float angle2, const float size2, float *size3)
{
    float x, y, rad_angle1, rad_angle2;

    rad_angle1 = (angle1 * M_PI) / 180;
    rad_angle2 = (angle2 * M_PI) / 180;
    x = size1 * cos(rad_angle1);
    y = size1 * sin(rad_angle1);
    x += size2 * cos(rad_angle2);
    y += size2 * sin(rad_angle2);
    *size3 = sqrt(x*x + y*y);
    if (x == 0) {
	if (y < 0)
	    return 270;
	if (y > 0)
	    return 90;
	return angle1;
    }
/*    if (y == 0) {
	if (x > 0)
	    return 0;
	else
	    return 180;
    }*/
    if (x > 0) {
	if (y < 0)
	    return (360 + ((atan(y/x) * 180) / M_PI));
	else
	    return ((atan(y/x) * 180) / M_PI);
    }
    else
	return (((atan(y/x) * 180) / M_PI) + 180);
}
/* End of these functions */

void
make_collision(const int ship1_number, const int ship2_number, const float distance)
{
    float delta_x, delta_y, alpha, beta, gamma, collision_speed;

    delta_x = ships_array[ship1_number]->ship.ship_position_left +
	ships_array[ship1_number]->ship.ship_distance_left -
	ships_array[ship2_number]->ship.ship_position_left -
	ships_array[ship2_number]->ship.ship_distance_left;
    delta_y = ships_array[ship1_number]->ship.ship_position_top +
	ships_array[ship1_number]->ship.ship_distance_top -
	ships_array[ship2_number]->ship.ship_position_top -
	ships_array[ship2_number]->ship.ship_distance_top;
    alpha = -atan2(delta_y, delta_x) + M_PI;
    while (alpha >= 2 * M_PI)
	alpha -= 2 * M_PI;
    while (alpha <= -2 * M_PI)
	alpha += 2 * M_PI;
#if DEBUG >= EXTRA_DEBUG
    printf("Angle alpha: %0.2f\n", (alpha * 180) / M_PI);
#endif /* DEBUG */
    beta = (ships_array[ship1_number]->ship.motion_angle * M_PI) / 180;	    /* Motion angle in radians */
    gamma = beta - alpha;
    if (gamma >= 2 * M_PI)
	gamma -= 2 * M_PI;
    else
	if (gamma <= 2 * M_PI)
	    gamma += 2 * M_PI;
    collision_speed = cos(gamma) * ships_array[ship1_number]->ship.ship_speed;
    if (collision_speed <= 0)
	return;
    beta = (ships_array[ship2_number]->ship.motion_angle * M_PI) / 180;	    /* Motion angle in radians */
    gamma = beta - alpha - M_PI;
    if (gamma >= 2 * M_PI)
	gamma -= 2 * M_PI;
    else
	if (gamma <= 2 * M_PI)
	    gamma += 2 * M_PI;
    beta = alpha + M_PI;
    if (beta > 2 * M_PI)
	beta -= 2 * M_PI;
    hit_ship(ship1_number, abs((int)collision_speed)/COLLISION_STRENGTH_CONST);
    hit_ship(ship2_number, abs((int)collision_speed)/COLLISION_STRENGTH_CONST);
#if DEBUG >= EXTRA_DEBUG
    printf("Motion vector ship1: <)%0.2f, ->%0.2f\n", (beta * 180) / M_PI, collision_speed);
    printf("Motion vector ship2: <)%0.2f, ->%0.2f\n", (alpha * 180) / M_PI, collision_speed);
#endif /* DEBUG */
    ships_array[ship1_number]->ship.motion_angle = get_angle_of_vectors_add(
	    ships_array[ship1_number]->ship.motion_angle, ships_array[ship1_number]->ship.ship_speed,
	    ((beta * 180) / M_PI), collision_speed, &ships_array[ship1_number]->ship.ship_speed);
    ships_array[ship2_number]->ship.motion_angle = get_angle_of_vectors_add(
	    ships_array[ship2_number]->ship.motion_angle, ships_array[ship2_number]->ship.ship_speed,
	    ((alpha * 180) / M_PI), collision_speed, &ships_array[ship2_number]->ship.ship_speed);
}

void
init_ships(const size_t max_count)
{
    ships_count = max_count;
    ships_array = (object_type **)malloc(sizeof(ship_type *) * ships_count);
    memset((void *)ships_array, 0, sizeof(ship_type *) * ships_count);
}

void
free_ships(void)
{
    int counter;

    for (counter = 0; counter < (int)ships_count; counter++)
	if (is_created_ship(counter))
	    delete_ship(counter);
}

void
delete_ship(const int ship_number)
{
    assert(is_created_ship(ship_number));
    SDL_FreeSurface(ships_array[ship_number]->ship.ship_image);
    SDL_FreeSurface(ships_array[ship_number]->ship.ship_shield_image);
    SDL_FreeSurface(ships_array[ship_number]->ship.normal_shot_image);
    SDL_FreeSurface(ships_array[ship_number]->ship.blast_shot_image);
    free(ships_array[ship_number]);
    ships_array[ship_number] = NULL;
}

int
is_created_ship(const int ship_number)
{
    assert(ship_number < ships_count);
    return (int)(ships_array[ship_number]);
}

int
is_destroyed_ship(const int ship_number)
{
    assert(ship_number < ships_count);
    assert(is_created_ship(ship_number));
    return (ships_array[ship_number]->ship.ship_armor < 0);
}

object_type *
create_ship(const int ship_number)
{
/*    char str[MAX_FILES_NAMES_LENGTH];*/
/*    Uint32 color;*/

    assert(!is_created_ship(ship_number));
    assert(ship_number < ships_count);
    ships_array[ship_number] = (object_type *)malloc(sizeof(ship_type));
    init_space_object(&ships_array[ship_number]->space_object);
    ships_array[ship_number]->type = SHIP_TYPE;
    ships_array[ship_number]->ship.options = DO_NOT_DRAW;
#if 0
    sprintf(str, SHIP_FILE_NAME, ship_number);
    ships_array[ship_number]->ship.ship_image = IMG_Load(get_file(str, image_file));
    sprintf(str, SHIP_SHIELD_FILE_NAME, ship_number);
#else /* true */
    ships_array[ship_number]->ship.ship_image = try_load_image(SHIP_FILE_NAME, ship_number, 1);
#endif /* true */
#if 0
#ifdef SHIELD_IS_BMP
    ships_array[ship_number]->ship.ship_shield_image = SDL_LoadBMP(get_file(str, image_file));
    color = SDL_MapRGB(ships_array[ship_number]->ship.ship_shield_image->format, 0, 0, 0);
    SDL_SetColorKey(ships_array[ship_number]->ship.ship_shield_image, SDL_SRCCOLORKEY, color);
#else /* !SHIELD_IS_BMP */
    ships_array[ship_number]->ship.ship_shield_image = IMG_Load(get_file(str));
#endif /* !SHIELD_IS_BMP */
#else /* true */
    ships_array[ship_number]->ship.ship_shield_image = try_load_image(SHIP_SHIELD_FILE_NAME, ship_number, 1);
#endif /* true */
#if 0
    sprintf(str, SHOT_FILE_NAME, ship_number);
    ships_array[ship_number]->ship.normal_shot_image = IMG_Load(get_file(str, image_file));
    sprintf(str, BLAST_FILE_NAME, ship_number);
    ships_array[ship_number]->ship.blast_shot_image = IMG_Load(get_file(str, image_file));
#else /* true */
    ships_array[ship_number]->ship.normal_shot_image = try_load_image(SHOT_FILE_NAME, ship_number, 1);
    ships_array[ship_number]->ship.blast_shot_image = try_load_image(BLAST_FILE_NAME, ship_number, 1);
#endif /* true */
    ships_array[ship_number]->ship.ship_angle = rand() % 360;
    ships_array[ship_number]->ship.motion_angle = ships_array[ship_number]->ship.ship_angle;
    ships_array[ship_number]->ship.ships_count = ships_array[ship_number]->ship.ship_image->h / ships_array[ship_number]->ship.ship_image->w;
    ships_array[ship_number]->ship.ship_position_left = rand() % screen_width;
    ships_array[ship_number]->ship.ship_position_top = rand() % screen_height;
    ships_array[ship_number]->ship.ship_shields = 100;
    ships_array[ship_number]->ship.ship_weapons = 100;
    ships_array[ship_number]->ship.ship_armor = 100;
    set_ship_system(ships_array[ship_number]->ship.systems.engine, 100, 25, 25);
    set_ship_system(ships_array[ship_number]->ship.systems.weapons, 100, 25, 25);
    set_ship_system(ships_array[ship_number]->ship.systems.shields, 100, 25, 25);
    set_ship_system(ships_array[ship_number]->ship.systems.controls, 100, 25, 25);

    return ships_array[ship_number];
}

size_t
get_max_ships_count(void)
{
    return ships_count;
}

float
get_ship_health(const int ship_number)
{
    assert(is_created_ship(ship_number));
    return ((float)ships_array[ship_number]->ship.ship_armor/1.0);
}

object_type *
get_ship(const int ship_number)
{
    assert(ship_number < ships_count);
    return ships_array[ship_number];
}

void
hit_ship(const int ship_number, const int damage)
{
    int rand_num;

    if (is_destroyed_ship(ship_number))
	return;
    if (ships_array[ship_number]->ship.ship_shields > 0)
	ships_array[ship_number]->ship.need_update |= NEED_REDRAW;
    ships_array[ship_number]->ship.ship_shields -= damage;
    if (ships_array[ship_number]->ship.ship_shields < 0) {
	ships_array[ship_number]->ship.ship_armor += ships_array[ship_number]->ship.ship_shields;
	if (ships_array[ship_number]->ship.ship_armor < 0) {
#if DEBUG >= EXTRA_DEBUG
	    printf("Ship %d is exploding, please evacuate!\n", ship_number);
#endif /* DEBUG */
	    create_explosion(ships_array[ship_number]);
	    return;
	}
	rand_num = rand() % NUM_OF_SHIP_SYSTEMS;
	switch (rand_num) {
	    case engine_system:
		ships_array[ship_number]->ship.systems.engine.condition += 2 * ships_array[ship_number]->ship.ship_shields;
		if (ships_array[ship_number]->ship.systems.engine.condition < 0)
		    ships_array[ship_number]->ship.systems.engine.condition = 0;
		correct_ship_speed(ship_number);
		break;
	    case weapons_system:
		ships_array[ship_number]->ship.systems.weapons.condition += 2 * ships_array[ship_number]->ship.ship_shields;
		if (ships_array[ship_number]->ship.systems.weapons.condition < 0)
		    ships_array[ship_number]->ship.systems.weapons.condition = 0;
		break;
	    case shields_system:
		ships_array[ship_number]->ship.systems.shields.condition += 2 * ships_array[ship_number]->ship.ship_shields;
		if (ships_array[ship_number]->ship.systems.shields.condition < 0)
		    ships_array[ship_number]->ship.systems.shields.condition = 0;
		break;
	    case controls_system:
		ships_array[ship_number]->ship.systems.controls.condition += 2 * ships_array[ship_number]->ship.ship_shields;
		if (ships_array[ship_number]->ship.systems.controls.condition < 0)
		    ships_array[ship_number]->ship.systems.controls.condition = 0;
		break;
	}
	ships_array[ship_number]->ship.ship_shields = 0;
#if DEBUG > NORMAL_DEBUG
	printf("Ship%d: A(%hd) SH(%f/%hd) WP(%f/%hd) EN(%f/%hd) CO(%f/%hd)\n",
		ship_number, (short int)ships_array[ship_number]->ship.ship_armor,
		ships_array[ship_number]->ship.ship_shields, (short int)ships_array[ship_number]->ship.systems.shields.condition,
		ships_array[ship_number]->ship.ship_weapons, (short int)ships_array[ship_number]->ship.systems.weapons.condition,
		ships_array[ship_number]->ship.ship_speed, (short int)ships_array[ship_number]->ship.systems.engine.condition,
		rotate_by_ship(ship_number), (short int)ships_array[ship_number]->ship.systems.controls.condition);
#endif /* DEBUG */
    }
}

void
correct_ship_speed(const int ship_number)
{
    float max_speed;

    assert(is_created_ship(ship_number));
    max_speed = (((Sint16)ships_array[ship_number]->ship.systems.engine.max_energy * (Sint16)ships_array[ship_number]->ship.systems.engine.condition) / 100);
    if (ships_array[ship_number]->ship.ship_speed > max_speed)
	ships_array[ship_number]->ship.ship_speed = max_speed;
}

float
rotate_by_ship(const int ship_number)
{
    float max_rotate;

    assert(is_created_ship(ship_number));
    max_rotate = (((Sint16)ships_array[ship_number]->ship.systems.controls.max_energy * (Sint16)ships_array[ship_number]->ship.systems.controls.condition) / 100);
    return max_rotate / ROTATE_CONST;
}

void
clean_ship(const int ship_number, SDL_Surface *to)
{
    assert(is_created_ship(ship_number));
    clean_space_object(&ships_array[ship_number]->space_object, to);
}

void
increase_ship_speed(const int ship_number)
{
    float max_speed, delta_speed, speed;

    assert(is_created_ship(ship_number));
    speed = ships_array[ship_number]->ship.ship_speed;
    max_speed = (((Sint16)ships_array[ship_number]->ship.systems.engine.max_energy * (Sint16)ships_array[ship_number]->ship.systems.engine.condition) / 100);
    delta_speed = max_speed / FROM_0_TO_100_ACCELERATE;
    ships_array[ship_number]->ship.motion_angle = get_angle_of_vectors_add(
	    ships_array[ship_number]->ship.ship_angle, delta_speed,
	    ships_array[ship_number]->ship.motion_angle, speed, &speed);
/*    speed += delta_speed;*/
    if (speed > max_speed)
	speed = max_speed;
    ships_array[ship_number]->ship.ship_speed = speed;
}

void
decrease_ship_speed(const int ship_number)
{
    float max_speed, delta_speed, speed;

    assert(is_created_ship(ship_number));
    speed = ships_array[ship_number]->ship.ship_speed;
    max_speed = (((Sint16)ships_array[ship_number]->ship.systems.engine.max_energy * (Sint16)ships_array[ship_number]->ship.systems.engine.condition) / 100);
    delta_speed = max_speed / FROM_0_TO_100_ACCELERATE;
    ships_array[ship_number]->ship.motion_angle = get_angle_of_vectors_add(
	    ships_array[ship_number]->ship.ship_angle + 180, delta_speed,
	    ships_array[ship_number]->ship.motion_angle, speed, &speed);
    if (speed > max_speed)
	speed = max_speed;
    ships_array[ship_number]->ship.ship_speed = speed;
}

int
set_ship_systems(const int ship_number, const Sint8 engine_max, const Sint8 weapons_max, const Sint8 shields_max, const Sint8 controls_max)
{
    assert(is_created_ship(ship_number));

    if (engine_max < 0 || weapons_max < 0 || shields_max < 0 || controls_max < 0 || (engine_max + weapons_max + shields_max + controls_max) > 100)
	return 0;
    ships_array[ship_number]->ship.systems.engine.max_energy = engine_max;
    ships_array[ship_number]->ship.systems.weapons.max_energy = weapons_max;
    ships_array[ship_number]->ship.systems.shields.max_energy = shields_max;
    ships_array[ship_number]->ship.systems.controls.max_energy = controls_max;
    return 1;
}

int
set_ship_repair(const int ship_number, const Sint8 engine_priority, const Sint8 weapons_priority, const Sint8 shields_priority, const Sint8 controls_priority)
{
    assert(is_created_ship(ship_number));

    if (engine_priority < 0 || weapons_priority < 0 || shields_priority < 0 || controls_priority < 0 || (engine_priority + weapons_priority + shields_priority + controls_priority) > 100)
	return 0;
    ships_array[ship_number]->ship.systems.engine.repair_speed = engine_priority;
    ships_array[ship_number]->ship.systems.weapons.repair_speed = weapons_priority;
    ships_array[ship_number]->ship.systems.shields.repair_speed = shields_priority;
    ships_array[ship_number]->ship.systems.controls.repair_speed = controls_priority;
    return 1;
}

void
rotate_ship(const int ship_number, const float delta_angle)
{
    float angle;

    ships_array[ship_number]->ship.need_update |= NEED_REDRAW;
    angle = ships_array[ship_number]->ship.ship_angle + delta_angle;
    while (angle < 0)
	angle += 360;
    while (angle >= 360)
	angle -= 360;
    if (ships_array[ship_number]->ship.ship_speed == 0)
	ships_array[ship_number]->ship.motion_angle = angle;
    ships_array[ship_number]->ship.ship_angle = angle;
}

void
repair_ship(const int ship_number)
{
    assert(is_created_ship(ship_number));
    if (ships_array[ship_number]->ship.systems.engine.condition) {
	ships_array[ship_number]->ship.systems.engine.condition += ships_array[ship_number]->ship.systems.engine.repair_speed;
	if (ships_array[ship_number]->ship.systems.engine.condition > 100)
	    ships_array[ship_number]->ship.systems.engine.condition = 100;
    }
    if (ships_array[ship_number]->ship.systems.weapons.condition) {
	ships_array[ship_number]->ship.systems.weapons.condition += ships_array[ship_number]->ship.systems.weapons.repair_speed;
	if (ships_array[ship_number]->ship.systems.weapons.condition > 100)
	    ships_array[ship_number]->ship.systems.weapons.condition = 100;
    }
    if (ships_array[ship_number]->ship.systems.shields.condition) {
	ships_array[ship_number]->ship.systems.shields.condition += ships_array[ship_number]->ship.systems.shields.repair_speed;
	if (ships_array[ship_number]->ship.systems.shields.condition > 100)
	    ships_array[ship_number]->ship.systems.shields.condition = 100;
    }
    if (ships_array[ship_number]->ship.systems.controls.condition) {
	ships_array[ship_number]->ship.systems.controls.condition += ships_array[ship_number]->ship.systems.controls.repair_speed;
	if (ships_array[ship_number]->ship.systems.controls.condition > 100)
	    ships_array[ship_number]->ship.systems.controls.condition = 100;
    }
}

int
redraw_ship_if_hit(const int ship_number, const SDL_Rect *rect)
{
    Sint16 x1, x2, y1, y2;

    assert(is_created_ship(ship_number));
    if (ships_array[ship_number]->ship.need_update & NEED_REDRAW)
	return -1;
    x1 = ships_array[ship_number]->ship.ship_position_left;
    y1 = ships_array[ship_number]->ship.ship_position_top;
    x2 = x1 + ships_array[ship_number]->ship.ship_image->w;
    y2 = y1 + ships_array[ship_number]->ship.ship_image->h / ships_array[ship_number]->ship.ships_count;
    if (((rect->x >= x1 && rect->x < x2) || (rect->x + rect->w > x1 && rect->x + rect->w <= x2) ||
	    (rect->x < x1 && rect->x + rect->w <= x2)) &&
	    ((rect->y >= y1 && rect->y < y2) ||
	     (rect->y + rect->h > x1 && rect->y + rect->h <= y2) ||
	     (rect->y < y1 && rect->y + rect->h <= y2))) {
	ships_array[ship_number]->ship.need_update |= NEED_REDRAW;
	return 1;
    }
    return 0;
}

void
draw_ship(const int ship_number, SDL_Surface *to)
{
    SDL_Rect imgrect, dstrect, shldrect, dstrect2;
    float angle;

    assert(is_created_ship(ship_number));
    if (ships_array[ship_number]->ship.is_shown)
	return;
    ships_array[ship_number]->ship.need_update &= ~NEED_REDRAW;
    imgrect.w = ships_array[ship_number]->ship.ship_image->w;
    imgrect.h = ships_array[ship_number]->ship.ship_image->h / ships_array[ship_number]->ship.ships_count;
    angle = ships_array[ship_number]->ship.ship_angle + 180 / ships_array[ship_number]->ship.ships_count;
    if (angle >= 360)
	angle -= 360;
    imgrect.x = 0;
    imgrect.y = ((((Uint16)angle) * ships_array[ship_number]->ship.ships_count) / 360) * imgrect.w;
    dstrect.x = ships_array[ship_number]->ship.ship_position_left - (imgrect.w / 2);
    dstrect.y = ships_array[ship_number]->ship.ship_position_top - (imgrect.h / 2);
    if (dstrect.x < 0) {
	imgrect.w += dstrect.x;
	imgrect.x -= dstrect.x;
	dstrect.x = 0;
    }
    if (dstrect.y < 0) {
	imgrect.h += dstrect.y;
	imgrect.y -= dstrect.y;
	dstrect.y = 0;
    }
    if (dstrect.x + imgrect.w >= screen_width)
	imgrect.w = screen_width - dstrect.x - 1;
    if (dstrect.y + imgrect.h >= screen_height)
	imgrect.h = screen_height - dstrect.y - 1;
    dstrect.w = imgrect.w;
    dstrect.h = imgrect.h;
    memcpy((void *)&shldrect, (void *)&imgrect, sizeof(SDL_Rect));
    if (shldrect.y >= ships_array[ship_number]->ship.ship_shield_image->h)
	shldrect.y = imgrect.y % ships_array[ship_number]->ship.ship_image->w;
    SDL_SetAlpha(ships_array[ship_number]->ship.ship_shield_image, SDL_SRCALPHA, SET_0_TRANSPARENT_100_OPAQUE((Uint16)ships_array[ship_number]->ship.ship_shields));
    memcpy((void *)&dstrect2, (void *)&dstrect, sizeof(SDL_Rect));
    memcpy((void *)&ships_array[ship_number]->ship.update_rect, (void *)&dstrect, sizeof(SDL_Rect));
    SDL_BlitSurface(ships_array[ship_number]->ship.ship_shield_image, &shldrect, to, &dstrect);
    memcpy((void *)&dstrect, (void *)&dstrect2, sizeof(SDL_Rect));
    SDL_BlitSurface(ships_array[ship_number]->ship.ship_image, &imgrect, to, &dstrect);
    /*DoCleanShip(ship_number, to);
    SDL_UpdateRects(to, 1, &dstrect);*/
    ships_array[ship_number]->ship.need_update |= DRAW_UPDATE;
    ships_array[ship_number]->ship.is_shown = 1;
}

int
check_collision(const int ship_number)
{
    int counter, distance, left_delta, top_delta, result, was;
    object_type *this_ship, *that_ship;

    was = 0;
    this_ship = get_ship(ship_number);
    for (counter = 0; counter < ships_count; counter++)
	if (counter != ship_number && is_created_ship(counter) && (!is_destroyed_ship(counter))) {
	    that_ship = get_ship(counter);
	    distance = this_ship->ship.ship_image->w / 2 + that_ship->ship.ship_image->w / 2;
	    left_delta = this_ship->ship.ship_position_left - that_ship->ship.ship_position_left;
	    top_delta = this_ship->ship.ship_position_top - that_ship->ship.ship_position_top;
	    if (abs(left_delta) < distance && abs(top_delta) < distance)
		if ((result = hypot(left_delta, top_delta)) < distance) {
/*		    was_collision(ship_number, counter);
		    hit_ship(counter, COLLISION_STRENGTH);
		    hit_ship(ship_number, COLLISION_STRENGTH);
		    this_ship->ship.ship_speed = 0;*/
		    make_collision(ship_number, counter, distance);
#if DEBUG > NORMAL_DEBUG
		    printf("Collision(%d).\n", ship_number);
#endif /* DEBUG */
		    was = 1;
		}
	}
    return was;
}

void
do_move_with_ship(const int ship_number, SDL_Surface *to)
{
    float fleft, ftop/*, delta_fleft, delta_ftop*/;
    Sint16 left, top/*, delta_left, delta_top*/;
    int collision;

    assert(is_created_ship(ship_number));
    assert(!is_destroyed_ship(ship_number));
    collision = 0;
    if (ships_array[ship_number]->ship.ship_shields < 100)
	ships_array[ship_number]->ship.need_update |= NEED_REDRAW;
    ships_array[ship_number]->ship.ship_weapons += ships_array[ship_number]->ship.systems.weapons.condition *
	ships_array[ship_number]->ship.systems.weapons.max_energy / (float)(100 * WEAPONS_CHARGE_CONST);
    if (ships_array[ship_number]->ship.ship_weapons > 100)
	ships_array[ship_number]->ship.ship_weapons = 100;
    ships_array[ship_number]->ship.ship_shields += ships_array[ship_number]->ship.systems.shields.condition *
	ships_array[ship_number]->ship.systems.shields.max_energy / (float)(100 * SHIELDS_CHARGE_CONST);
    if (ships_array[ship_number]->ship.ship_shields > 100)
	ships_array[ship_number]->ship.ship_shields = 100;
    if (ships_array[ship_number]->ship.need_update & NEED_REDRAW)
	clean_ship(ship_number, to);
    left = ships_array[ship_number]->ship.ship_position_left;
    fleft = ships_array[ship_number]->ship.ship_distance_left;
    top = ships_array[ship_number]->ship.ship_position_top;
    ftop = ships_array[ship_number]->ship.ship_distance_top;
    do_move_with_space_object(&ships_array[ship_number]->space_object, to, stop_object);
/*    delta_left = ships_array[ship_number]->ship.ship_position_left - left;
    delta_fleft = ships_array[ship_number]->ship.ship_distance_left - fleft;
    delta_top = ships_array[ship_number]->ship.ship_position_top - top;
    delta_ftop = ships_array[ship_number]->ship.ship_distance_top - ftop;
    ships_array[ship_number]->ship.ship_position_left = left;
    ships_array[ship_number]->ship.ship_distance_left = fleft;
    ships_array[ship_number]->ship.ship_position_top = top;
    ships_array[ship_number]->ship.ship_distance_top = ftop;*/
    if (check_collision(ship_number)) {
	ships_array[ship_number]->ship.ship_position_left = left;
	ships_array[ship_number]->ship.ship_distance_left = fleft;
	ships_array[ship_number]->ship.ship_position_top = top;
	ships_array[ship_number]->ship.ship_distance_top = ftop;
    }
/*    if (check_collision(ship_number)) {
	ships_array[ship_number]->ship.ship_position_left -= delta_left;
	ships_array[ship_number]->ship.ship_position_top -= delta_top;
	ships_array[ship_number]->ship.ship_distance_left -= delta_fleft;
	ships_array[ship_number]->ship.ship_distance_top -= delta_ftop;
	collision = 1;
    }
    else {
	ships_array[ship_number]->ship.ship_position_left += delta_left;
	ships_array[ship_number]->ship.ship_position_top += delta_top;
	ships_array[ship_number]->ship.ship_distance_left += delta_fleft;
	ships_array[ship_number]->ship.ship_distance_top += delta_ftop;
    }*/
/*    if (check_collision(ship_number)) {
	if (collision) {
	    ships_array[ship_number]->ship.ship_position_left += delta_left / 2;
	    ships_array[ship_number]->ship.ship_position_top += delta_top / 2;
	    ships_array[ship_number]->ship.ship_distance_left += delta_fleft / 2;
	    ships_array[ship_number]->ship.ship_distance_top += delta_ftop / 2;
	}
	else {
	    ships_array[ship_number]->ship.ship_position_left -= delta_left;
	    ships_array[ship_number]->ship.ship_position_top -= delta_top;
	    ships_array[ship_number]->ship.ship_distance_left -= delta_fleft;
	    ships_array[ship_number]->ship.ship_distance_top -= delta_ftop;
	}
    }*/
    draw_ship(ship_number, to);
}

void
do_move_with_all_ships(SDL_Surface *to)
{
    int counter;
    
    for (counter = 0; counter < (int)ships_count; counter++)
	if (is_created_ship(counter)) {
	    if (!is_destroyed_ship(counter))
		do_move_with_ship(counter, to);
	    else {
		clean_ship(counter, to);
		purge_ship(counter, to);
	    }
	}
    counter = 0;
/*    while (counter < (int)ships_count)
	if (is_destroyed_ship(counter))
	    delete_ship(counter);
	else
	    counter++;*/
}

void
update_ship(const int ship_number, SDL_Surface *to)
{
    update_space_object(&ships_array[ship_number]->space_object, to);
}

void
purge_ship(const int ship_number, SDL_Surface *to)
{
    assert(is_created_ship(ship_number));
    purge_space_object(&ships_array[ship_number]->space_object, to);
}

void
update_all_ships(SDL_Surface *to)
{
    int counter;

    for (counter = 0; counter < (int)ships_count; counter++)
	    if (is_created_ship(counter)) {
		if (is_destroyed_ship(counter))
		    clean_ship(counter, to);
		purge_ship(counter, to);
	    }
    for (counter = 0; counter < (int)ships_count; counter++)
	    if (is_created_ship(counter) && !is_destroyed_ship(counter))
		update_ship(counter, to);
}

void
repair_all_ships(void)
{
    int counter;

    for (counter = 0; counter < (int)ships_count; counter++)
	    if (is_created_ship(counter) && !is_destroyed_ship(counter))
		repair_ship(counter);
}
