#ifndef SHIP_H
#define SHIP_H

#include <SDL/SDL.h>
#include "objects.h"

#define NUM_OF_SHIP_SYSTEMS		4

void init_ships(const size_t max_count);
void free_ships(void);
void delete_ship(const int ship_number);
int is_created_ship(const int ship_number);
int is_destroyed_ship(const int ship_number);
object_type *create_ship(const int ship_number);
object_type *get_ship(const int ship_number);
size_t get_max_ships_count(void);
float get_ship_health(const int ship_number);
void hit_ship(const int ship_number, const int damage);
void correct_ship_speed(const int ship_number);
float rotate_by_ship(const int ship_number);
void increase_ship_speed(const int ship_number);
void decrease_ship_speed(const int ship_number);
int set_ship_systems(const int ship_number, const Sint8 engine_max, const Sint8 weapons_max, const Sint8 shields_max, const Sint8 controls_max);
int set_ship_repair(const int ship_number, const Sint8 engine_priority, const Sint8 weapons_priority, const Sint8 shields_priority, const Sint8 controls_priority);
void clean_ship(const int ship_number, SDL_Surface *to);
void purge_ship(const int ship_number, SDL_Surface *to);
void draw_ship(const int ship_number, SDL_Surface *to);
void update_ship(const int ship_number, SDL_Surface *to);
void rotate_ship(const int ship_number, const float delta_angle);
void repair_ship(const int ship_number);
int redraw_ship_if_hit(const int ship_number, const SDL_Rect *rect);
void do_move_with_ship(const int ship_number, SDL_Surface *to);
void do_move_with_all_ships(SDL_Surface *to);
int check_collision(const int ship_number);
void was_collision(const int ship_number1, const int ship_number2);
void update_all_ships(SDL_Surface *to);
void repair_all_ships(void);
void make_collision(const int ship1_number, const int ship2_number, const float distance);
	
#endif /* SHIP_H */
