#include <assert.h>
#include <stdlib.h>
#include <math.h>
#include "shot.h"
#include "objects.h"
#include "debug.h"
#if DEBUG > NO_DEBUG
#   include <stdio.h>
#endif /* DEBUG */


#define INC_SIZE		8

#define SHOT_POWER		6
#define SHOT_COSTS		5
#define SHOT_SPEED		40
#define NORMAL_SHOT_LIFE	40
#define BLAST_SHOT_LIFE		60
#define BLAST_POWER(energy)	((11 * energy) / 9)
#define MIN_BLAST_SHOT_CONSTS	10

#define DRAW_UPDATE		1
#define CLEAN_UPDATE		2

#define hypot(x,y)			sqrt((x) * (x) + (y) * (y))

/*struct shot_record **all_shots;
size_t all_shots_size, all_shots_count;*/

/*
 * Constants definition:
 */
#define INIT_SIZE		64

object_type **all_shots;
size_t all_shots_size;
int all_shots_count;
extern Sint16 screen_width, screen_height;

void
init_shots_array(void)
{
    all_shots_size = INIT_SIZE;
    all_shots_count = 0;
    all_shots = (object_type **)malloc(all_shots_size * sizeof(object_type *));
    memset((void *)all_shots, 0, all_shots_size * sizeof(object_type *));
}

void
free_shots_array(void)
{
    delete_all_shots();
    free(all_shots);
}

object_type *
create_and_init_shot(object_type *owner, const shot_strength_type type)
{
    object_type *new_record;
    float angle, delta_fleft, delta_ftop;

    if (owner->ship.ship_weapons < (type == normal_shot ? SHOT_COSTS : MIN_BLAST_SHOT_CONSTS))
	return NULL;
    if (all_shots_count == all_shots_size) {
	all_shots_size += INC_SIZE;
	all_shots = (object_type **)realloc((void *)all_shots, all_shots_size * sizeof(object_type *));
	memset((void *)(all_shots + all_shots_count), 0, (all_shots_size - all_shots_count) * sizeof(object_type *));
    }
    new_record = (object_type *)malloc(sizeof(shot_type));
    init_space_object(&new_record->space_object);
    new_record->type = SHOT_TYPE;
    new_record->shot.shot_image = (type == normal_shot ? owner->ship.normal_shot_image : owner->ship.blast_shot_image);
    new_record->shot.shot_strenght = (type == normal_shot ? SHOT_POWER : BLAST_POWER(owner->ship.ship_weapons));
    owner->ship.ship_weapons = (type == normal_shot ? owner->ship.ship_weapons - SHOT_COSTS : 0);
    new_record->shot.shot_speed = SHOT_SPEED + owner->ship.ship_speed / SPEED_CONST;
    new_record->shot.shot_life = (type == normal_shot ? NORMAL_SHOT_LIFE : BLAST_SHOT_LIFE);

    new_record->shot.shot_angle = owner->ship.ship_angle;
    new_record->shot.shot_position_left = owner->ship.ship_position_left;
    new_record->shot.shot_position_top = owner->ship.ship_position_top;
    if (new_record->shot.shot_angle == 0 || new_record->shot.shot_angle == 180) {
	delta_ftop = 0;
	delta_fleft = (new_record->shot.shot_angle == 180 ? -1 : 1) * (owner->ship.ship_image->w / 2 + owner->ship.ship_speed / SPEED_CONST + 2 + new_record->shot.shot_image->w / 2);
    }
    else
	if (new_record->shot.shot_angle == 90 || new_record->shot.shot_angle == 270) {
	    delta_ftop = (new_record->shot.shot_angle == 90 ? -1 : 1) * (owner->ship.ship_image->w / 2 + owner->ship.ship_speed /SPEED_CONST + 2 + new_record->shot.shot_image->w / 2);
	    delta_fleft = 0;
	}
	else {
		angle = new_record->shot.shot_angle * M_PI / (float)180;
		delta_fleft = (owner->ship.ship_speed / SPEED_CONST + 2 + new_record->shot.shot_image->w / 2 + owner->ship.ship_image->w / 2) * cos(angle);
		delta_ftop = - (owner->ship.ship_speed / SPEED_CONST + 2 + new_record->shot.shot_image->w / 2 + owner->ship.ship_image->w / 2) * sin(angle);
	}
    delta_fleft += owner->ship.ship_position_left;
    delta_ftop += owner->ship.ship_position_top;
    if (delta_fleft > screen_width)
	delta_fleft = screen_width - 1;
    else
	if (delta_fleft < 0)
	    delta_fleft = 0;
    if (delta_ftop > screen_height)
	delta_ftop = screen_height - 1;
    else
	if (delta_ftop < 0)
	    delta_ftop = 0;

    new_record->shot.shot_position_left = delta_fleft;
    new_record->shot.shot_position_top = delta_ftop;
/*    new_record->shot.shot_distance_left = 0;
    new_record->shot.shot_distance_top = 0;
    new_record->shown = 0;
    new_record->update = 0;*/
    all_shots[all_shots_count++] = new_record;

    return new_record;
}

void
draw_shot(const int shot_number, SDL_Surface *to)
{
    assert(shot_number < all_shots_count);
    assert(all_shots[shot_number]);
    draw_space_object(&all_shots[shot_number]->space_object, to);
}

void
update_shot(const int shot_number, SDL_Surface *to)
{
    assert(shot_number < all_shots_count);
    assert(all_shots[shot_number]);
    update_space_object(&all_shots[shot_number]->space_object, to);
}

void
clean_shot(const int shot_number, SDL_Surface *to)
{
    assert(shot_number < all_shots_count);
    assert(all_shots[shot_number]);
    clean_space_object(&all_shots[shot_number]->space_object, to);
}

void
purge_shot(const int shot_number, SDL_Surface *to)
{
    assert(shot_number < all_shots_count);
    assert(all_shots[shot_number]);
    purge_space_object(&all_shots[shot_number]->space_object, to);
}

void
delete_dead_shots(SDL_Surface *to)
{
    int counter;
    
    for (counter = 0; counter < all_shots_count; counter++)
	while (all_shots[counter]->shot.shot_life == 0 || all_shots[counter]->shot.shot_speed == 0) {
	    clean_shot(counter, to);
	    purge_shot(counter, to);
	    free(all_shots[counter]);
	    all_shots[counter] = NULL;
	    all_shots_count--;
	    if (counter >= all_shots_count)
		return;
	    all_shots[counter] = all_shots[all_shots_count];
	    all_shots[all_shots_count] = NULL;
	}
}

void
update_all_shots(SDL_Surface *to)
{
    int counter;

    for (counter = 0; counter < all_shots_count; counter++)
	update_shot(counter, to);
    for (counter = 0; counter < all_shots_count; counter++)
	purge_shot(counter, to);
}

void
delete_all_shots(void)
{
    while (all_shots_count)
	if (all_shots[--all_shots_count])
	    free(all_shots[all_shots_count]);
}

int
check_hit(const int shot_number)
{
    int counter, distance, left_delta, top_delta, result;
    object_type *this_ship;
    size_t max_ships;

    assert(shot_number < all_shots_count);
    max_ships = get_max_ships_count();
    for (counter = 0; counter < (int)max_ships; counter++)
	if (is_created_ship(counter) && (!is_destroyed_ship(counter))) {
	    this_ship = get_ship(counter);
	    distance = this_ship->ship.ship_image->w / 2 + all_shots[shot_number]->shot.shot_image->w / 2;
	    left_delta = this_ship->ship.ship_position_left - all_shots[shot_number]->shot.shot_position_left;
	    top_delta = this_ship->ship.ship_position_top - all_shots[shot_number]->shot.shot_position_top;
	    if (abs(left_delta) < distance && abs(top_delta) < distance && ((result = (int)hypot((double)left_delta, (double)top_delta)) < distance)) {
/*		if ((this_ship->ship_image->w / 2) > sqrt(X_2(this_ship->ship_position_left - all_shots[shot_number]->shot_position_left) + X_2(this_ship->ship_position_top - all_shots[shot_number]->shot_position_top))) { */
		hit_ship(counter, all_shots[shot_number]->shot.shot_strenght);
#if DEBUG > NORMAL_DEBUG
		printf("Hit %d,%d:%d < %d.\n", left_delta, top_delta, result, distance);
#endif /* DEBUG */
		all_shots[shot_number]->shot.shot_life = 0;
		return 1;
	    }
	}
    return 0;
}

void
do_move_with_shot(const int shot_number, SDL_Surface *to)
{
/*    float angle, delta_fleft, delta_ftop;
    Sint16 delta_left, delta_top;*/

    assert(shot_number < all_shots_count);
    assert(all_shots[shot_number]);
    if (check_hit(shot_number))
	return;
/*    angle = all_shots[shot_number]->shot.shot_angle;
    if (angle == 0 || angle == 180) {
	delta_ftop = all_shots[shot_number]->shot.shot_distance_top;
	delta_fleft = (angle == 180 ? -1 : 1) * all_shots[shot_number]->shot.shot_speed + all_shots[shot_number]->shot.shot_distance_left;
    }
    else
	if (angle == 90 || angle == 270) {
	    delta_ftop = (angle == 90 ? -1 : 1) * all_shots[shot_number]->shot.shot_speed + all_shots[shot_number]->shot.shot_distance_top;
	    delta_fleft = all_shots[shot_number]->shot.shot_distance_left;
	}
	else {
		angle *= M_PI / (float)180;
		delta_fleft = all_shots[shot_number]->shot.shot_speed * cos(angle) + all_shots[shot_number]->shot.shot_distance_left;
		delta_ftop = - all_shots[shot_number]->shot.shot_speed * sin(angle) + all_shots[shot_number]->shot.shot_distance_top;
	}
    delta_left = delta_fleft;
    delta_top = delta_ftop;
    all_shots[shot_number]->shot.shot_distance_left = delta_fleft - delta_left;
    all_shots[shot_number]->shot.shot_distance_top = delta_ftop - delta_top;
    MoveShot(shot_number, delta_left, delta_top, to);*/
    do_move_with_space_object(&all_shots[shot_number]->space_object, to, bounce_object);
/*    change_space_object_position(&all_shots[shot_number]->space_object, delta_left, delta_top, to, bounce_object);*/
    all_shots[shot_number]->shot.shot_life--;
    if (check_hit(shot_number))
	return;
/*    draw_shot(shot_number, to);*/
}

/*void
MoveShot(const int shot_number, const Sint16 delta_left, const Sint16 delta_top, SDL_Surface *to)
{
    Sint16 left, top;

    left = all_shots[shot_number]->shot.shot_position_left;
    top = all_shots[shot_number]->shot.shot_position_top;
    left += delta_left;
    top += delta_top;
    if (left < 0) {
	left = 0;
	all_shots[shot_number]->shot.shot_life = 1;
    }
    if (top < 0) {
	top = 0;
	all_shots[shot_number]->shot.shot_life = 1;
    }
    if (left > screen_width) {
	left = screen_width;
	all_shots[shot_number]->shot.shot_life = 1;
    }
    if (top > screen_height) {
	top = screen_height;
	all_shots[shot_number]->shot.shot_life = 1;
    }
    all_shots[shot_number]->shot.shot_position_left = left;
    all_shots[shot_number]->shot.shot_position_top = top;
}*/

void
do_move_with_all_shots(SDL_Surface *to)
{
    int counter;
    
    delete_dead_shots(to);
/*    for (counter = 0; counter < all_shots_count; counter++)
	clean_shot(counter, to);*/
    for (counter = 0; counter < all_shots_count; counter++)
	do_move_with_shot(counter, to);
}
