#include <stdio.h>
#include <stdlib.h>
#include "terminal.h"
#include "debug.h"
#include "config.h"
#include "ship.h"

SDLKey
get_user_key(const player_actions what)
{
    SDLKey key;

    printf("Press your ");
    switch (what) {
	case speed_up:
	    printf("speed up");
	    break;
	case speed_down:
	    printf("speed down");
	    break;
	case rotate_left:
	    printf("rotate left");
	    break;
	case rotate_right:
	    printf("rotate right");
	    break;
	case fire:
	    printf("fire");
	    break;
	case fire_blast:
	    printf("fire blast");
	    break;
    }
    printf(" key: ");
    fflush(stdout);
    key = get_key();
    printf("%s\n", SDL_GetKeyName(key));
    return key;
}

void
print_user_key(const int ship_number, const player_actions what)
{
    switch (what) {
	case speed_up:
	    printf("Speed up");
	    break;
	case speed_down:
	    printf("Speed down");
	    break;
	case rotate_left:
	    printf("Rotate left");
	    break;
	case rotate_right:
	    printf("Rotate right");
	    break;
	case fire:
	    printf("Fire");
	    break;
	case fire_blast:
	    printf("Fire blast");
	    break;
    }
    printf("'s key is: %s.\n", SDL_GetKeyName(get_action_key(ship_number, what)));
    
}

void
set_system_settings(const int ship_number)
{
    char result;
    Sint16 engn, wpn, shld, cntrl;

    engn = system_settings[ship_number][engine_system];
    wpn = system_settings[ship_number][weapons_system];
    shld = system_settings[ship_number][shields_system];
    cntrl = system_settings[ship_number][controls_system];
    printf("ENG=%hd WPN=%hd SHL=%hd CNT=%hd\n", engn, wpn, shld, cntrl);
    printf("Do you want to change systems settings? [y/N]");
    scanf("%c", &result);
    if (result != '\n')
	getc(stdin);
    if (result == 'y' || result == 'Y') {
	printf("Set maximum energy for engine [0-100]%%:");
	scanf("%hd", &engn);
	getc(stdin);
	printf("Set maximum energy for weapons [0-%d]%%:", 100 - engn);
	scanf("%hd", &wpn);
	getc(stdin);
	printf("Set maximum energy for shields [0-%d]%%:", 100 - engn - wpn);
	scanf("%hd", &shld);
	getc(stdin);
	printf("Set maximum energy for controls [0-%d]%%:", 100 - engn - wpn - shld);
	scanf("%hd", &cntrl);
	getc(stdin);
    }
    if (set_ship_systems(ship_number, engn, wpn, shld, cntrl)) {
	printf("Systems were modified.\n");
	system_settings[ship_number][engine_system] = engn;
	system_settings[ship_number][weapons_system] = wpn;
	system_settings[ship_number][shields_system] = shld;
	system_settings[ship_number][controls_system] = cntrl;
	return;
    }
    else
	printf("Not correct data, restoring defaults.\n");
}

void
set_repair_settings(const int ship_number)
{
    char result;
    Sint16 engn, wpn, shld, cntrl;

    engn = repair_settings[ship_number][engine_system];
    wpn = repair_settings[ship_number][weapons_system];
    shld = repair_settings[ship_number][shields_system];
    cntrl = repair_settings[ship_number][controls_system];
    printf("ENG=%hd WPN=%hd SHL=%hd CNT=%hd\n", engn, wpn, shld, cntrl);
    printf("Do you want to change repair speed settings? [y/N]");
    scanf("%c", &result);
    if (result != '\n')
	getc(stdin);
    if (result == 'y' || result == 'Y') {
	printf("Set maximum energy for engine [0-100]%%:");
	scanf("%hd", &engn);
	getc(stdin);
	printf("Set maximum energy for weapons [0-%d]%%:", 100 - engn);
	scanf("%hd", &wpn);
	getc(stdin);
	printf("Set maximum energy for shields [0-%d]%%:", 100 - engn - wpn);
	scanf("%hd", &shld);
	getc(stdin);
	printf("Set maximum energy for controls [0-%d]%%:", 100 - engn - wpn - shld);
	scanf("%hd", &cntrl);
	getc(stdin);
    }
    if (set_ship_systems(ship_number, engn, wpn, shld, cntrl)) {
	printf("Repair speeds were modified.\n");
	repair_settings[ship_number][engine_system] = engn;
	repair_settings[ship_number][weapons_system] = wpn;
	repair_settings[ship_number][shields_system] = shld;
	repair_settings[ship_number][controls_system] = cntrl;
	return;
    }
    else
	printf("Not correct data, restoring defaults.\n");
}

int
get_answer(const int default_answer)
{
    char result;

    scanf("%c", &result);
    if (result == '\n')
	return default_answer;
    getc(stdin);
    return (int)result;
}

void
set_game_settings(void)
{
    int counter, players, again, im, counter2, joys;
    char result;
    
    printf("How many players do you want[1-%d]: ", maximum_ships_count);
    scanf("%d", &players);
    getc(stdin);
    if (players > maximum_ships_count)
	players = maximum_ships_count;
    if (players < 1)
	players = 1;
    if (current_ships_count) {
	system_settings = (settings_type *)realloc((void *)system_settings, players * sizeof(settings_type));
	repair_settings = (settings_type *)realloc((void *)repair_settings, players * sizeof(settings_type));
    } else {
	system_settings = (settings_type *)malloc(players * sizeof(settings_type));
	repair_settings = (settings_type *)malloc(players * sizeof(settings_type));
    }
    for (; current_ships_count < players; current_ships_count++) {
	system_settings[current_ships_count][engine_system] = 25;
	system_settings[current_ships_count][weapons_system] = 25;
	system_settings[current_ships_count][shields_system] = 25;
	system_settings[current_ships_count][controls_system] = 25;
	repair_settings[current_ships_count][engine_system] = 25;
	repair_settings[current_ships_count][weapons_system] = 25;
	repair_settings[current_ships_count][shields_system] = 25;
	repair_settings[current_ships_count][controls_system] = 25;
    }
    for (counter = 0; counter < players; counter++) {
	create_ship(counter);
	do {
	    again = 0;
	    
	    printf("Player %d settings:\n", counter + 1);

	    im = get_players_input_method(counter);
	    printf("Input method: %s\n", get_input_method_desc(im));

	    switch (im) {
		case keyboard:
		    print_user_key(counter, speed_up);
		    print_user_key(counter, speed_down);
		    print_user_key(counter, rotate_left);
		    print_user_key(counter, rotate_right);
		    print_user_key(counter, fire);
		    print_user_key(counter, fire_blast);
		    break;
#ifdef ENABLE_JOYSTICK
		case joystick:
		    printf("Joystick number %d (%s) selected.\nFire is button number %d and blast fire is button number %d.\n",
			    get_joystick(counter), SDL_JoystickName(get_joystick(counter)), get_action_joy_button(counter, fire),
			    get_action_joy_button(counter, fire));
		    break;
#endif /* ENABLE_JOYSTICK */
	    }

	    printf("Do you want to change this? [y/N]");
	    result = get_answer((int)'N');
	    if (result == 'y' || result == 'Y') {
		printf("Select input method [1-Keyboard/2-Joystick]: ");
		result = get_answer((int)'1');
		switch (result) {
		    case 'k':
		    case 'K':
		    case '1':
			set_players_input_method(counter, keyboard);
			printf("Move cursor to "PACKAGE"'s window and then follow instructions.\n");
			set_action_key(counter, speed_up, get_user_key(speed_up));
			set_action_key(counter, speed_down, get_user_key(speed_down));
			set_action_key(counter, rotate_left, get_user_key(rotate_left));
			set_action_key(counter, rotate_right, get_user_key(rotate_right));
			set_action_key(counter, fire, get_user_key(fire));
			set_action_key(counter, fire_blast, get_user_key(fire_blast));
			break;
#ifdef ENABLE_JOYSTICK
		    case 'j':
		    case 'J':
		    case '2':
			joys = SDL_NumJoysticks();
			if (joys > 0) {
			    set_players_input_method(counter, joystick);
			    printf("Select joystick [1-%d]: ", SDL_NumJoysticks());
			    for (counter2 = 0; counter2 < SDL_NumJoysticks(); counter2++)
				printf("%d. %s\n", counter2 + 1, SDL_JoystickName(counter2));
			    scanf("%c", &result);
			    getc(stdin);
			    set_joystick(counter, (Uint8)result);
			    printf("Select button for fire action: ");
			    scanf("%c", &result);
			    getc(stdin);
			    set_action_joy_button(counter, fire, (Uint8)result);
			    printf("Select button for fire blast action: ");
			    scanf("%c", &result);
			    getc(stdin);
			    set_action_joy_button(counter, fire_blast, (Uint8)result);
			}
#if DEBUG>=NORMAL_DEBUG
			else
			    fprintf(stderr, "No joystick found!\n");
#endif /* DEBUG */
			break;
#endif /* ENABLE_JOYSTICK */
		    default:
			set_players_input_method(counter, no_input);
		}
	    }

	    set_system_settings(counter);
	    set_repair_settings(counter);

	    printf("Do you accept this settings? [Y/n/q]");
	    result = get_answer((int)'Y');
	    if (result == 'n' || result == 'N')
		again = 1;
	    if (result == 'q' || result == 'Q')
		exit(EXIT_SUCCESS);
	} while (again);
    }
}

int
continue_question(void)
{
    int result;

    printf("Do you want to continue? [Y/n]");
    result = get_answer((int)'Y');
    if (result == 'n' || result == 'N')
	return 0;
    else
	return 1;
}
