#include <stdio.h>
#include <SDL.h>
#include <math.h>
#include <time.h>
#include "background.h"
#include "common.h"
#include "options.h"
#include "map.h"
#include "ship.h"

void
test_draw_background(SDL_Surface *to, const int count)
{
    int counter, count_digits, counter_digits, counter2, max;
    Uint16 temp;
    SDL_Rect rect;
    SDL_Color red = { 0xFF, 0x0, 0x0, 0 };
    time_t start_time, stop_time;
    
    count_digits = (int)log10(count) + 1;
    printf("Testing draw_background (whole screen): %*d/%d", count_digits, 0, count);
    start_time = time(NULL);
    for (counter = 1; counter <= count; counter++) {
	counter_digits = (int)log10(counter) + 1;
	draw_background(to, NULL);
	for (counter2 = 0; counter2 < counter_digits + count_digits + 1; counter2++)
	    printf("\b");
	printf("%d/%d", counter, count);
	fflush(stdout);
    }
    stop_time = time(NULL);
    printf(" @ %.5f sec (%.5f sec).\n", difftime(stop_time, start_time), difftime(stop_time, start_time) / count);

    printf("Testing draw_background (inside one tile): %*d/%d", count_digits, 0, count);
    rect.x = 200;
    rect.y = 200;
    rect.w = 40;
    rect.h = 40;
    start_time = time(NULL);
    for (counter = 1; counter <= count; counter++) {
	counter_digits = (int)log10(counter) + 1;
	draw_background(to, &rect);
	for (counter2 = 0; counter2 < counter_digits + count_digits + 1; counter2++)
	    printf("\b");
	printf("%d/%d", counter, count);
	fflush(stdout);
    }
    stop_time = time(NULL);
    printf(" @ %.5f sec (%.5f sec).\n", difftime(stop_time, start_time), difftime(stop_time, start_time) / count);

    printf("Testing draw_background (parts of four tiles): %*d/%d", count_digits, 0, count);
    rect.x = 100;
    rect.y = 100;
    rect.w = 40;
    rect.h = 40;
    start_time = time(NULL);
    for (counter = 1; counter <= count; counter++) {
	counter_digits = (int)log10(counter) + 1;
	draw_background(to, &rect);
	for (counter2 = 0; counter2 < counter_digits + count_digits + 1; counter2++)
	    printf("\b");
	printf("%d/%d", counter, count);
	fflush(stdout);
    }
    stop_time = time(NULL);
    printf(" @ %.5f sec (%.5f sec).\n", difftime(stop_time, start_time), difftime(stop_time, start_time) / count);

    printf("Testing draw_background (not whole screen): %*d/%d", count_digits, 0, count);
    rect.x = 100;
    rect.y = 100;
    rect.w = 540;
    rect.h = 380;
    start_time = time(NULL);
    for (counter = 1; counter <= count; counter++) {
	counter_digits = (int)log10(counter) + 1;
	draw_background(to, &rect);
	for (counter2 = 0; counter2 < counter_digits + count_digits + 1; counter2++)
	    printf("\b");
	printf("%d/%d", counter, count);
	fflush(stdout);
    }
    stop_time = time(NULL);
    printf(" @ %.5f sec (%.5f sec).\n", difftime(stop_time, start_time), difftime(stop_time, start_time) / count);
    printf("Testing what draw_background draws ... ");
    SDL_FillRect(to, NULL, SDL_MapRGB(to->format, 0, 0, 0));
    SDL_Flip(to);
    rect.w = 7;
    rect.h = 6;
    rect.x = 0;
    rect.y = 0;
    max = (count > 200 ? 200 : count);
    for (counter = 0; counter < max; counter++) {
	draw_background(to, &rect);
	draw_rect(to, &rect, &red);
	temp = rect.h;
	rect.h = rect.w;
	rect.w = temp + 1;
	rect.x = (rect.x + 13) % to->w;
	rect.y = (rect.y + 7) % to->h;
    }
    SDL_Flip(to);
    printf("done.\n");
}

void
test_map(const int count)
{
    object_type *ship, *ptr;
    map_walk_type walk;
    int counter, count_digits, counter_digits, counter2;
    SDL_Rect rect1, rect2;
    time_t start_time, stop_time;

    count_digits = (int)log10(count) + 1;
    ship = create_ship(0);
    rect1.x = 20;
    rect1.w = 100;
    rect1.y = 40;
    rect1.h = 120;
    printf("Testing insert_object_into_map&delete_object_from_map: %*d/%d", count_digits, 0, count);
    start_time = time(NULL);
    for (counter = 1; counter <= count; counter++) {
	counter_digits = (int)log10(counter) + 1;
	insert_object_into_map(&rect1, ship);
	delete_object_from_map(&rect1, ship);
	for (counter2 = 0; counter2 < counter_digits + count_digits + 1; counter2++)
	    printf("\b");
	printf("%d/%d", counter, count);
	fflush(stdout);
    }
    stop_time = time(NULL);
    ptr = get_first_object_from_map(&rect1, &walk);
    if (ptr != NULL) {
	fprintf(stderr, "Error: Other object than NULL [ship(%p)] found: %p !!!\n", ship, ptr);
	while ((ptr = get_next_object_from_map(&walk)))
	    fprintf(stderr, "Error: Next object [ship(%p)] found: %p !!!\n", ship, ptr);
    }
    printf(" @ %.5f sec (%.5f sec).\n", difftime(stop_time, start_time), difftime(stop_time, start_time) / count);
    rect2.w = rect1.w;
    rect2.h = rect1.h;
    rect2.x = 50;
    rect2.y = 80;
    printf("Testing move_with_object_in_map: %*d/%d", count_digits, 0, count);
    insert_object_into_map(&rect1, ship);
    start_time = time(NULL);
    for (counter = 1; counter <= count; counter++) {
	counter_digits = (int)log10(counter) + 1;
	move_with_object_in_map(&rect1, &rect2, ship);
	move_with_object_in_map(&rect2, &rect1, ship);
	for (counter2 = 0; counter2 < counter_digits + count_digits + 1; counter2++)
	    printf("\b");
	printf("%d/%d", counter, count);
	ptr = get_first_object_from_map(&rect2, &walk);
	if (ptr != ship)
	    fprintf(stderr, "Error: Other object than NULL [ship(%p)] found: %p !!!\n", ship, ptr);
	fflush(stdout);
    }
    stop_time = time(NULL);
    rect2.x = rect2.y = 0;
    rect2.w = screen_width;
    rect2.h = screen_height;
    ptr = get_first_object_from_map(&rect2, &walk);
    if (ptr != ship)
	fprintf(stderr, "Error: Other object than ship(%p) found: %p !!!\n", ship, ptr);
    while ((ptr = get_next_object_from_map(&walk))) {
	fprintf(stderr, "Error: Next object [ship(%p)] found: %p !!!\n", ship, ptr);
    }
    delete_object_from_map(&rect1, ship);
    printf(" @ %.5f sec (%.5f sec).\n", difftime(stop_time, start_time), difftime(stop_time, start_time) / count);
    delete_ship(0);
}
